package org.amse.chekh.paint_graph.abstact_view.impl;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.amse.chekh.paint_graph.abstact_view.IAbstractView;
import org.amse.chekh.paint_graph.abstact_view.IEvent;
import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.amse.chekh.paint_graph.abstact_view.IListener;

/**
 *     
 * 
 * @author 
 */
class AbstractView implements IAbstractView {

	private static final long serialVersionUID = 1L;

	/**
	 *      
	 */
	private List<IGraph> myGraphs;

	/**
	 *   
	 */
	private List<IListener> myListeners;

	/**
	 * ,   
	 */
	AbstractView() {
		this.myGraphs = new ArrayList<IGraph>();
		this.myListeners = new LinkedList<IListener>();
	}

	public void addGraph(IGraph graph) {
		this.myGraphs.add(graph);
		this.fireEvent(new Event("addGraph"));
	}
	
	public void addGraphAt(int index, IGraph graph) {
		this.myGraphs.add(index, graph);
		this.fireEvent(new Event("addGraphAt"));
	}

	public void addAllGraphs(List<IGraph> graphs) {
		this.myGraphs.addAll(graphs);
		this.fireEvent(new Event("addAllGraphs"));
	}

	public void removeGraph(IGraph graph) {
		this.myGraphs.remove(graph);
		this.fireEvent(new Event("removeGraph"));
	}

	public void changeGraph(IGraph oldGraph, IGraph newGraph) {
		int index = this.myGraphs.indexOf(oldGraph);
		this.myGraphs.remove(oldGraph);
		this.myGraphs.add(index, newGraph);
		this.fireEvent(new Event("changeGraph"));
	}

	public void addListener(IListener l) {
		this.myListeners.add(l);
	}

	public void removeListener(IListener l) {
		this.myListeners.remove(l);
	}

	public Object getElementAt(int index) {
		return this.myGraphs.get(index);
	}

	public int getSize() {
		return this.myGraphs.size();
	}

	/**
	 *     
	 * 
	 * @param e -
	 *             
	 */
	private void fireEvent(IEvent e) {
		for (IListener l : this.myListeners) {
			l.processEvent(e);
		}
	}

	public void changeGraphSelected(IGraph graph) {
		((Graph) graph).changeGraphSelected();
	}

	public void setGraphPaint(IGraph graph, boolean isPaint) {
		((Graph) graph).setGraphPaint(isPaint);
		this.fireEvent(new Event("changeGraphPaint"));
	}

	public void setProperties(IGraph graph, double minX, double maxX,
			int countOfSegment, Color color) {
		((Graph) graph).setProperties(minX, maxX, countOfSegment, color);
		this.fireEvent(new Event("changeGrapgProperties"));
	}

	public List<IGraph> graphs() {
		return Collections.unmodifiableList(this.myGraphs);
	}

}
