package org.amse.chekh.paint_graph.io.writer;

import java.awt.Color;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.List;

import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.amse.chekh.paint_graph.abstact_view.IAbstractView;
import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *     
 * 
 * @author 
 * 
 */
public class Writer {

	/**
	 *  ,   
	 */
	private String myFileName;

	/**
	 *  ,  
	 */
	private IAbstractView myAbstractView;

	private String myMinX;

	private String myMaxX;

	private String myMinY;

	private String myMaxY;

	/**
	 *        
	 * 
	 * @param fileName -
	 *              ,   
	 * @param abstractView -
	 *             ,   
	 */
	public Writer(String fileName, IAbstractView abstractView, double minX,
			double maxX, double minY, double maxY) {
		this.myFileName = fileName;
		this.myAbstractView = abstractView;
		this.myMinX = minX + "";
		this.myMaxX = maxX + "";
		this.myMinY = minY + "";
		this.myMaxY = maxY + "";
	}

	/**
	 *      .   - , 
	 *   ,   
	 * 
	 * @exception ParserConfigurationException
	 * @exception IOException
	 * @exception TransformerException
	 */
	public void write() throws ParserConfigurationException, IOException,
			TransformerException {

		List<IGraph> graphs = this.myAbstractView.graphs();

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document doc = db.newDocument();

		Element root = doc.createElement("graphs");
		Element minX = doc.createElement("minimum_Visible_X");
		minX.setAttribute("value", this.myMinX);
		Element maxX = doc.createElement("maximum_Visible_X");
		maxX.setAttribute("value", this.myMaxX);
		Element minY = doc.createElement("minimum_Visible_Y");
		minY.setAttribute("value", this.myMinY);
		Element maxY = doc.createElement("maximum_Visible_Y");
		maxY.setAttribute("value", this.myMaxY);
		root.appendChild(minX);
		root.appendChild(maxX);
		root.appendChild(minY);
		root.appendChild(maxY);
		root.setAttribute("count", graphs.size() + "");
		doc.appendChild(root);

		int i = 0;
		for (IGraph g : graphs) {
			Element gr = doc.createElement("graph");
			gr.setAttribute("number", i + "");
			gr.setAttribute("function", "Y=" + g.getFunction().getText());
			Element properties = doc.createElement("properties");
			minX = doc.createElement("minimum_X");
			minX.setAttribute("value", g.getProperties().getMinimumX() + "");
			maxX = doc.createElement("maximum_X");
			maxX.setAttribute("value", g.getProperties().getMaximumX() + "");
			Element countOfSegment = doc.createElement("count_of_segment");
			countOfSegment.setAttribute("value", g.getProperties()
					.getCountOfSegment()
					+ "");
			Element colorEl = doc.createElement("color");
			Color color = g.getProperties().getColor();
			String s;
			if (color == Color.BLACK) {
				s = "black";
			} else if (color == Color.RED) {
				s = "red";
			} else if (color == Color.GREEN) {
				s = "green";
			} else if (color == Color.BLUE) {
				s = "blue";
			} else if (color == Color.YELLOW) {
				s = "yellow";
			} else {
				s = "";
			}
			colorEl.setAttribute("value", s);
			properties.appendChild(minX);
			properties.appendChild(maxX);
			properties.appendChild(countOfSegment);
			properties.appendChild(colorEl);
			gr.appendChild(properties);
			Element paintGraph = doc.createElement("isPaint");
			paintGraph.setAttribute("value", g.isGraphPaint() + "");
			gr.appendChild(paintGraph);
			root.appendChild(gr);
			i++;
		}

		DOMSource domSource = new DOMSource(doc);
		TransformerFactory trFactory = TransformerFactory.newInstance();
		trFactory.setAttribute("indent-number", new Integer(4));
		Transformer transformer = trFactory.newTransformer();

		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		FileOutputStream fs = new FileOutputStream(new File(this.myFileName));

		Result r = new StreamResult(new OutputStreamWriter(fs));

		transformer.transform(domSource, r);
		fs.close();

	}

}
