package org.amse.chekh.paint_graph.main;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.List;
import java.util.Stack;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;

import javax.xml.parsers.*;
import javax.xml.transform.*;

import org.xml.sax.SAXException;

import org.amse.chekh.paint_graph.abstact_view.IAbstractView;
import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.amse.chekh.paint_graph.abstact_view.IGraphProperties;
import org.amse.chekh.paint_graph.abstact_view.impl.AbstractViewFactory;
import org.amse.chekh.paint_graph.abstact_view.impl.GraphFactory;
import org.amse.chekh.paint_graph.io.reader.Reader;
import org.amse.chekh.paint_graph.io.writer.Writer;
import org.amse.chekh.paint_graph.model.IFunction;
import org.amse.chekh.paint_graph.view.dialogs.AddFunctionDialog;
import org.amse.chekh.paint_graph.view.dialogs.ChangeVisibleIntervalDialog;
import org.amse.chekh.paint_graph.view.dialogs.DifferencialDialog;
import org.amse.chekh.paint_graph.view.dialogs.IntegralDialog;
import org.amse.chekh.paint_graph.view.dialogs.TangentOrNormalDialog;
import org.amse.chekh.paint_graph.view.ListForFunctions;
import org.amse.chekh.paint_graph.view.PaintPanel;

/**
 *   
 * 
 * @author 
 */
public class MainUI extends JFrame {

	private static final long serialVersionUID = 1L;

	public static final String ICON_PATH = "org/amse/chekh/paint_graph/icon/";

	/**
	 *    
	 */
	private final JFileChooser myFileChooser = new JFileChooser();

	private final FileFilter PAINT_GRAPH_FILTER = new PaintGraphFileFilter();

	private final FileFilter IMAGE_FILTER = new ImageFileFilter();

	private boolean myDocumentIsSaved = true;

	private String fileName = "";

	private final double deltaX = 2.0;

	private final double deltaY = 2.0;

	/**
	 *      
	 */
	private PaintPanel myPaintPanel;

	/**
	 *  
	 */
	private ListForFunctions myFunctionsList;

	/**
	 *    
	 */
	private AddFunctionDialog myAddFormulaDialog = new AddFunctionDialog(this);

	/**
	 *         
	 */
	private TangentOrNormalDialog myTangentOrNormalDialog = new TangentOrNormalDialog(
			this);

	/**
	 *        
	 */
	private DifferencialDialog myDifferencialDialog = new DifferencialDialog(
			this);

	/**
	 *          
	 */
	private IntegralDialog myIntegralDialog = new IntegralDialog(this);

	/**
	 *      
	 */
	private ChangeVisibleIntervalDialog myVisibleIntervalDialog = new ChangeVisibleIntervalDialog(
			this);

	/**
	 *   
	 */
	private JButton mySaveButton;

	private JMenuItem mySaveItem;

	/**
	 *    
	 */
	private JButton myDeleteButton;

	private JMenuItem myDeleteItem;

	/**
	 *      
	 */
	private JButton myTangentOrNormalButton;

	private JMenuItem myTangentOrNormalItem;

	private JButton myRedoButton;

	private JButton myUndoButton;

	private Stack<CompliteActions> myUndoStack = new Stack<CompliteActions>();

	private Stack<CompliteActions> myRedoStack = new Stack<CompliteActions>();

	/**
	 *    
	 */
	private JToggleButton myScaleButton;

	/**
	 * ,   
	 */
	public MainUI() {
		super("PaintGraph");

		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

		this.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {
				if (MainUI.this.confirm()) {
					MainUI.this.dispose();
				}
			}

		});

		IAbstractView abstractView = AbstractViewFactory.createAbsractView();
		this.myPaintPanel = new PaintPanel(abstractView);
		this.myFunctionsList = new ListForFunctions(abstractView);

		Box graph = Box.createHorizontalBox();
		graph.setBorder(new EmptyBorder(12, 12, 12, 12));

		JPanel panelForListFunction = new JPanel(new BorderLayout());
		panelForListFunction.setBorder(new TitledBorder(" "));
		JScrollPane scrPane = new JScrollPane(this.myFunctionsList);
		scrPane.setBorder(null);
		panelForListFunction.add(scrPane, BorderLayout.CENTER);

		JButton enter = new JButton(" ");
		enter.addActionListener(new AddAction("Add"));

		JPanel p = new JPanel(new BorderLayout());
		p.add(enter, BorderLayout.SOUTH);
		p.add(panelForListFunction, BorderLayout.CENTER);

		p.setPreferredSize(new Dimension(200, 500));
		p.setMaximumSize(new Dimension(200, p.getMaximumSize().height));

		this.myFunctionsList.addMouseListener(new FunctionListMouseListener());

		this.myFunctionsList.addKeyListener(new FunctionListKeyListener());

		this.myFunctionsList
				.addListSelectionListener(new ListSelectionListener() {

					public void valueChanged(ListSelectionEvent e) {
						JList list = (JList) e.getSource();
						if (list.isSelectionEmpty()) {
							MainUI.this.myDeleteButton.setEnabled(false);
							MainUI.this.myDeleteItem.setEnabled(false);
						} else {
							MainUI.this.myDeleteButton.setEnabled(true);
							MainUI.this.myDeleteItem.setEnabled(true);
						}
					}

				});

		this.myFunctionsList.getModel().addListDataListener(
				new ListDataListener() {

					public void intervalRemoved(ListDataEvent e) {
						if (((ListModel) e.getSource()).getSize() == 0) {
							MainUI.this.myTangentOrNormalButton
									.setEnabled(false);
							MainUI.this.myTangentOrNormalItem.setEnabled(false);
						}
					}

					public void intervalAdded(ListDataEvent e) {
						MainUI.this.myTangentOrNormalButton.setEnabled(true);
						MainUI.this.myTangentOrNormalItem.setEnabled(true);
					}

					public void contentsChanged(ListDataEvent e) {
						if (((ListModel) e.getSource()).getSize() == 0) {
							MainUI.this.myTangentOrNormalButton
									.setEnabled(false);
							MainUI.this.myTangentOrNormalItem.setEnabled(false);
						} else {
							MainUI.this.myTangentOrNormalButton
									.setEnabled(true);
							MainUI.this.myTangentOrNormalItem.setEnabled(true);
						}
					}

				});

		PaintPanelMouseListener listener = new PaintPanelMouseListener();
		this.myPaintPanel.addMouseMotionListener(listener);
		this.myPaintPanel.addMouseListener(listener);

		this.myPaintPanel.setPreferredSize(new Dimension(500, 500));

		graph.add(p);
		graph.add(Box.createHorizontalStrut(5));
		graph.add(this.myPaintPanel);

		this.setJMenuBar(this.crateMenuBar());
		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(createToolBar(), BorderLayout.NORTH);
		this.getContentPane().add(graph, BorderLayout.CENTER);

		this.myFunctionsList.setBackground(this.myPaintPanel.getBackground());
		this.myFileChooser.addChoosableFileFilter(PAINT_GRAPH_FILTER);
		this.myFileChooser.addChoosableFileFilter(IMAGE_FILTER);

		pack();

		setVisible(true);

	}

	private JToolBar createToolBar() {
		JToolBar tb = new JToolBar();
		tb.add(new NewAction("New"));
		tb.add(new OpenAction("Open"));
		this.mySaveButton = tb.add(new SaveAction("Save"));
		this.mySaveButton.setEnabled(false);
		tb.addSeparator();
		tb.add(new AddAction("Add"));
		this.myDeleteButton = tb.add(new DeleteAction("Delete"));
		this.myDeleteButton.setEnabled(false);
		tb.addSeparator();
		tb.add(new DifferencialAction("Differentation"));
		this.myTangentOrNormalButton = tb.add(new TangentOrNormalAction(
				"Tangent"));
		this.myTangentOrNormalButton.setEnabled(false);
		tb.add(new IntegralAction("Integration"));
		tb.addSeparator();
		this.myUndoButton = tb.add(new UndoAction("Undo"));
		this.myUndoButton.setEnabled(false);
		this.myRedoButton = tb.add(new RedoAction("Redo"));
		this.myRedoButton.setEnabled(false);
		tb.addSeparator();
		tb.add(new LeftAction("Left"));
		tb.add(new RightAction("Right"));
		tb.add(new UpAction("Up"));
		tb.add(new DownAction("Down"));
		tb.addSeparator();
		this.myScaleButton = new JToggleButton(new ImageIcon(this
				.getURL(MainUI.ICON_PATH + "MashtabPlus.png")));
		tb.add(this.myScaleButton);
		tb.addSeparator();
		tb.add(new ChangeVisibleIntervalAction("Limits"));
		return tb;
	}

	/**
	 *    
	 * 
	 * @return    
	 */
	private JMenuBar crateMenuBar() {
		JMenuBar menu = new JMenuBar();
		menu.add(this.createFileMenu());
		menu.add(this.createGraphicsMenu());
		return menu;
	}

	/**
	 *   File
	 * 
	 * @return   File
	 */
	private JMenu createFileMenu() {
		final JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);
		fileMenu.add(new JMenuItem(new NewAction("New")));
		fileMenu.addSeparator();
		fileMenu.add(new JMenuItem(new OpenAction("Open")));
		this.mySaveItem = fileMenu.add(new JMenuItem(new SaveAction("Save")));
		this.mySaveItem.setEnabled(false);
		fileMenu.add(new JMenuItem(new SaveAsAction("Save as ... ")));

		final JMenuItem saveAsPicture = new JMenuItem("Save as picture");

		saveAsPicture.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {

				MainUI.this.myFileChooser
						.setDialogTitle("  ");
				MainUI.this.myFileChooser
						.setFileSelectionMode(JFileChooser.FILES_ONLY);
				MainUI.this.myFileChooser.setFileFilter(IMAGE_FILTER);
				int res = MainUI.this.myFileChooser.showSaveDialog(MainUI.this);
				if (res == JFileChooser.APPROVE_OPTION) {
					Dimension size = MainUI.this.myPaintPanel.getSize();
					BufferedImage image = new BufferedImage(size.width,
							size.height, BufferedImage.TYPE_INT_RGB);
					MainUI.this.myPaintPanel.paint(image.getGraphics());
					String name = MainUI.this.myFileChooser.getSelectedFile()
							.getPath();
					if (!MainUI.this.myFileChooser.getSelectedFile().getName()
							.contains(".")) {
						name += ".png";
					}
					try {
						ImageIO.write(image, "png", new File(name));
					} catch (IOException ex) {
						JOptionPane.showMessageDialog(MainUI.this,
								"   !", "",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}

		});
		fileMenu.add(saveAsPicture);
		fileMenu.addSeparator();

		JMenuItem exit = new JMenuItem("Exit");
		exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
				KeyEvent.CTRL_MASK));
		exit.setMnemonic(KeyEvent.VK_X);
		exit.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				MainUI.this.dispose();
			}

		});
		fileMenu.add(exit);
		return fileMenu;
	}

	/**
	 *   Graphics
	 * 
	 * @return   Graphics
	 */
	private JMenu createGraphicsMenu() {
		final JMenu graphMenu = new JMenu("Graphics");
		graphMenu.setMnemonic(KeyEvent.VK_G);
		graphMenu.add(new JMenuItem(new AddAction("Add")));
		this.myDeleteItem = graphMenu.add(new JMenuItem(new DeleteAction(
				"Delete")));
		this.myDeleteItem.setEnabled(false);
		graphMenu.addSeparator();
		graphMenu.add(new DifferencialAction("Differentation"));
		this.myTangentOrNormalItem = graphMenu.add(new JMenuItem(
				new TangentOrNormalAction("Tangent or Normal")));
		this.myTangentOrNormalItem.setEnabled(false);
		graphMenu.addSeparator();
		graphMenu.add(new JMenuItem(new IntegralAction("Integration")));
		return graphMenu;
	}

	/**
	 *       
	 */
	private void open() {
		this.myFileChooser.setDialogTitle("...");
		this.myFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		this.myFileChooser.setFileFilter(PAINT_GRAPH_FILTER);
		this.myFileChooser.setSelectedFile(new File(this.fileName));
		int res = MainUI.this.myFileChooser.showOpenDialog(MainUI.this);
		if (res == JFileChooser.APPROVE_OPTION) {
			try {
				Reader r = new Reader(this.myFileChooser.getSelectedFile()
						.getPath());
				List<IGraph> graphs = r.read();
				IAbstractView abstractView = AbstractViewFactory
						.createAbsractView();
				DefaultListModel listModel = ((DefaultListModel) this.myFunctionsList
						.getModel());
				listModel.removeAllElements();
				this.myFunctionsList.setAbstractView(abstractView);
				this.myPaintPanel.setAbstractView(abstractView);
				this.myPaintPanel.setMinimunVisibleX(r.getMinimunVisibleX());
				this.myPaintPanel.setMaximunVisibleX(r.getMaximunVisibleX());
				this.myPaintPanel.setMinimunVisibleY(r.getMinimunVisibleY());
				this.myPaintPanel.setMaximunVisibleY(r.getMaximunVisibleY());
				this.deselectUndoAndRedo();
				for (IGraph g : graphs) {
					listModel.addElement(g);
				}
				abstractView.addAllGraphs(graphs);
				this.myDocumentIsSaved = true;
				this.mySaveButton.setEnabled(false);
				this.mySaveItem.setEnabled(false);
				this.fileName = this.myFileChooser.getSelectedFile().getPath();
			} catch (IOException ex) {
				JOptionPane.showMessageDialog(this, "  !",
						"", JOptionPane.ERROR_MESSAGE);
			} catch (SAXException ex) {
				JOptionPane.showMessageDialog(this,
						"    .   !",
						"", JOptionPane.ERROR_MESSAGE);
			} catch (ParserConfigurationException ex) {
				JOptionPane.showMessageDialog(this,
						"    .   !",
						"", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	/**
	 *     
	 */
	private boolean save(boolean isSaveAs) {
		String fileName;
		if (isSaveAs || this.fileName.equals("")) {
			this.myFileChooser.setDialogTitle(" ");
			this.myFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			this.myFileChooser.setSelectedFile(new File(this.fileName));
			this.myFileChooser.setFileFilter(PAINT_GRAPH_FILTER);
			int res = MainUI.this.myFileChooser.showSaveDialog(MainUI.this);
			if (res != JFileChooser.APPROVE_OPTION) {
				return false;
			}
			fileName = this.myFileChooser.getSelectedFile().getPath();
			if (!this.myFileChooser.getSelectedFile().getName().contains(".")) {
				fileName += ".pgr";
			}
		} else {
			fileName = this.fileName;
		}
		try {
			new Writer(fileName, this.myPaintPanel.getAbstractView(),
					this.myPaintPanel.getMinimunVisibleX(), this.myPaintPanel
							.getMaximunVisibleX(), this.myPaintPanel
							.getMinimunVisibleY(), this.myPaintPanel
							.getMaximunVisibleY()).write();
			this.deselectUndoAndRedo();
			this.myDocumentIsSaved = true;
			this.mySaveButton.setEnabled(false);
			this.mySaveItem.setEnabled(false);
			this.fileName = fileName;
			return true;
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(this, "   !",
					"", JOptionPane.ERROR_MESSAGE);
		} catch (ParserConfigurationException ex) {
			JOptionPane.showMessageDialog(this, " !", "",
					JOptionPane.ERROR_MESSAGE);
		} catch (TransformerException ex) {
			JOptionPane.showMessageDialog(this, " !", "",
					JOptionPane.ERROR_MESSAGE);
		}
		return false;
	}

	private void deselectUndoAndRedo() {
		this.myUndoStack = new Stack<CompliteActions>();
		this.myRedoStack = new Stack<CompliteActions>();
		this.myRedoButton.setEnabled(false);
		this.myUndoButton.setEnabled(false);
	}

	private void changeGraph(int index, JList list) {
		final DefaultListModel listModel = (DefaultListModel) list.getModel();
		final IGraph graph = (IGraph) listModel.getElementAt(index);
		final IFunction function = graph.getFunction();
		this.myAddFormulaDialog.setFunction(function.getText());
		this.myAddFormulaDialog.setProperties(graph.getProperties());
		IFunction newFunction = myAddFormulaDialog.createFunction(true);
		if (newFunction != null) {
			if (!function.getText().equals(newFunction.getText())) {
				boolean isPaint = graph.isGraphPaint();
				IGraph newGraph = GraphFactory.createGraph(newFunction,
						this.myAddFormulaDialog.getMinX(),
						this.myAddFormulaDialog.getMaxX(),
						this.myAddFormulaDialog.getCountOfSegment(),
						this.myAddFormulaDialog.getColor(), isPaint);
				listModel.removeElement(graph);
				listModel.add(index, newGraph);
				this.myPaintPanel.getAbstractView()
						.changeGraph(graph, newGraph);
				this.myUndoStack.push(new ChangeGraphCompliteAction(graph,
						newGraph));
			} else {
				IGraphProperties oldProperties = graph.getProperties();
				this.myPaintPanel.getAbstractView().setProperties(graph,
						this.myAddFormulaDialog.getMinX(),
						this.myAddFormulaDialog.getMaxX(),
						this.myAddFormulaDialog.getCountOfSegment(),
						this.myAddFormulaDialog.getColor());
				this.myUndoStack.push(new ChangePropertiesCompliteAction(graph,
						oldProperties, graph.getProperties()));
				this.myUndoButton.setEnabled(true);
			}
			this.myRedoStack = new Stack<CompliteActions>();
			this.myRedoButton.setEnabled(false);
			this.myDocumentIsSaved = false;
			this.mySaveButton.setEnabled(true);
			this.mySaveItem.setEnabled(true);
		}
	}

	private void addGraph(IFunction function) {
		if (function != null) {
			IGraph graph = GraphFactory.createGraph(function,
					myAddFormulaDialog.getMinX(), myAddFormulaDialog.getMaxX(),
					myAddFormulaDialog.getCountOfSegment(), myAddFormulaDialog
							.getColor(), true);
			DefaultListModel listModel = ((DefaultListModel) this.myFunctionsList
					.getModel());
			listModel.addElement(graph);
			this.myPaintPanel.getAbstractView().addGraph(graph);
			this.myUndoStack.push(new AddCompliteAction(graph));
			this.myRedoStack = new Stack<CompliteActions>();
			this.myRedoButton.setEnabled(false);
			this.myUndoButton.setEnabled(true);
			this.myDocumentIsSaved = false;
			this.mySaveButton.setEnabled(true);
			this.mySaveItem.setEnabled(true);
		}
	}

	private void deleteGraph(IGraph graph) {
		DefaultListModel listModel = (DefaultListModel) this.myFunctionsList
				.getModel();
		int index = listModel.indexOf(graph);
		listModel.removeElement(graph);
		this.myPaintPanel.getAbstractView().removeGraph(graph);
		this.myUndoStack.push(new RemoveCompliteAction(index, graph));
		this.myRedoStack = new Stack<CompliteActions>();
		this.myRedoButton.setEnabled(false);
		this.myUndoButton.setEnabled(true);
		this.myDocumentIsSaved = false;
		this.mySaveButton.setEnabled(true);
		this.mySaveItem.setEnabled(true);
	}

	/**
	 *      
	 */
	private class FunctionListKeyListener extends KeyAdapter {

		@Override
		public void keyPressed(KeyEvent e) {
			final JList list = (JList) e.getSource();
			final int index = list.getSelectedIndex();
			if (e.getKeyCode() == KeyEvent.VK_DELETE) {
				if (index != -1) {
					DefaultListModel listModel = (DefaultListModel) list
							.getModel();
					MainUI.this.deleteGraph((IGraph) listModel
							.getElementAt(index));
				}
			} else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
				if (index != -1) {
					MainUI.this.changeGraph(index, list);
					list.setSelectedIndex(index);
				}
			}
		}

	}

	/**
	 *      
	 */
	private class FunctionListMouseListener extends MouseAdapter {

		public void mousePressed(MouseEvent e) {
			final JList list = (JList) e.getSource();
			final int index = list.locationToIndex(e.getPoint());
			final DefaultListModel listModel = (DefaultListModel) list
					.getModel();
			if (e.getClickCount() == 1 && SwingUtilities.isLeftMouseButton(e)) {
				Point p = e.getPoint();
				final ListCellRenderer r = list.getCellRenderer();
				Rectangle cellBounds = list.getCellBounds(index, index);
				if (index != -1 && r != null && cellBounds != null
						&& cellBounds.contains(p)) {
					final IGraph graph = (IGraph) listModel.get(index);
					final ListSelectionModel lsm = list.getSelectionModel();
					Component component = r
							.getListCellRendererComponent(list, graph, index,
									lsm.isSelectedIndex(index), (list
											.hasFocus() && (lsm
											.getLeadSelectionIndex() == index)));
					component.setSize(cellBounds.width, cellBounds.height);
					component = component.getComponentAt(
							(int) (p.getX() - cellBounds.getX()), (int) (p
									.getX() - cellBounds.getX()));
					if (component instanceof JCheckBox) {
						final boolean isPaint = graph.isGraphPaint();
						MainUI.this.myPaintPanel.getAbstractView()
								.setGraphPaint(graph, !isPaint);
						MainUI.this.myUndoStack
								.push(new ChangeGraphPaintCompliteAction(graph));
						MainUI.this.myRedoStack = new Stack<CompliteActions>();
						MainUI.this.myRedoButton.setEnabled(false);
						MainUI.this.myUndoButton.setEnabled(true);
						MainUI.this.myDocumentIsSaved = false;
						MainUI.this.mySaveButton.setEnabled(true);
						MainUI.this.mySaveItem.setEnabled(true);
					}
				}
			} else if (e.getClickCount() == 2
					&& SwingUtilities.isLeftMouseButton(e)) {
				if (index == -1) {
					return;
				}
				MainUI.this.changeGraph(index, list);
			}
		}

	}

	/**
	 *       
	 */
	private class PaintPanelMouseListener extends MouseMotionAdapter implements
			MouseListener {

		private boolean myMouseIsDragged = false;

		@Override
		public void mouseMoved(MouseEvent e) {
			MainUI.this.myPaintPanel.updateGraphSelection(e.getPoint());
		}

		@Override
		public void mouseDragged(MouseEvent e) {
			if (MainUI.this.myScaleButton.isSelected()) {
				this.myMouseIsDragged = true;
				MainUI.this.myPaintPanel.setRectPaint(true);
				MainUI.this.myPaintPanel.setEndPointForScale(e.getPoint());
				MainUI.this.myPaintPanel.repaint();
			}
		}

		public void mouseClicked(MouseEvent e) {
		}

		public void mouseEntered(MouseEvent e) {
			if (MainUI.this.myScaleButton.isSelected()) {
				MainUI.this.myPaintPanel.setCursor(new Cursor(
						Cursor.CROSSHAIR_CURSOR));
			} else {
				MainUI.this.myPaintPanel.setCursor(new Cursor(
						Cursor.DEFAULT_CURSOR));
			}
		}

		public void mouseExited(MouseEvent e) {
			MainUI.this.myPaintPanel.deselectedAllGraph();
		}

		public void mousePressed(MouseEvent e) {
			if (MainUI.this.myScaleButton.isSelected()) {
				MainUI.this.myPaintPanel.setBeginPointForScale(e.getPoint());
			}
		}

		public void mouseReleased(MouseEvent e) {
			if (this.myMouseIsDragged) {
				MainUI.this.myPaintPanel.setRectPaint(false);
				this.myMouseIsDragged = false;
				double oldMinX = MainUI.this.myPaintPanel.getMinimunVisibleX();
				double oldMaxX = MainUI.this.myPaintPanel.getMaximunVisibleX();
				double oldMinY = MainUI.this.myPaintPanel.getMinimunVisibleY();
				double oldMaxY = MainUI.this.myPaintPanel.getMaximunVisibleY();
				MainUI.this.myPaintPanel.setEndPointForScale(e.getPoint());
				MainUI.this.myPaintPanel.changePaintRectangle();
				double newMinX = MainUI.this.myPaintPanel.getMinimunVisibleX();
				double newMaxX = MainUI.this.myPaintPanel.getMaximunVisibleX();
				double newMinY = MainUI.this.myPaintPanel.getMinimunVisibleY();
				double newMaxY = MainUI.this.myPaintPanel.getMaximunVisibleY();
				MainUI.this.myUndoStack.push(new ChangePropertiesPaintPanel(
						oldMinX, oldMaxX, oldMinY, oldMaxY, newMinX, newMaxX,
						newMinY, newMaxY));
				MainUI.this.myRedoStack = new Stack<CompliteActions>();
				MainUI.this.myRedoButton.setEnabled(false);
				MainUI.this.myUndoButton.setEnabled(true);
				MainUI.this.mySaveButton.setEnabled(true);
				MainUI.this.mySaveItem.setEnabled(true);
			}
		}
	}

	private URL getURL(String fileName) {
		URLClassLoader urlLoader = (URLClassLoader) this.getClass()
				.getClassLoader();
		URL fileLocation = urlLoader.findResource(fileName);
		return fileLocation;
	}

	private boolean confirm() {
		if (this.myDocumentIsSaved) {
			return true;
		}
		int res = JOptionPane.showConfirmDialog(this,
				"  ! ?", "",
				JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
		if (res == JOptionPane.CANCEL_OPTION) {
			return false;
		}
		if (res == JOptionPane.YES_OPTION) {
			return this.save(false);
		}
		return true;
	}

	/**
	 *       
	 */
	private class NewAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private NewAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke
					.getKeyStroke(KeyEvent.VK_N, KeyEvent.CTRL_MASK));
			this.putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_N);
			this.putValue(AbstractAction.SHORT_DESCRIPTION,
					"  ");
		}

		public void actionPerformed(ActionEvent e) {
			if (MainUI.this.confirm()) {
				IAbstractView abstractView = AbstractViewFactory
						.createAbsractView();
				((DefaultListModel) MainUI.this.myFunctionsList.getModel())
						.removeAllElements();
				MainUI.this.myPaintPanel.setAbstractView(abstractView);
				MainUI.this.myFunctionsList.setAbstractView(abstractView);
				MainUI.this.myPaintPanel.setMinimunVisibleX(-10.0);
				MainUI.this.myPaintPanel.setMaximunVisibleX(10.0);
				MainUI.this.myPaintPanel.setMinimunVisibleY(-10.0);
				MainUI.this.myPaintPanel.setMaximunVisibleY(10.0);
				MainUI.this.deselectUndoAndRedo();
				MainUI.this.fileName = "";
				MainUI.this.myDocumentIsSaved = true;
				MainUI.this.mySaveButton.setEnabled(false);
				MainUI.this.mySaveItem.setEnabled(false);
				MainUI.this.fileName = "";
				MainUI.this.repaint();
			}
		}
	}

	/**
	 *        
	 */
	private class OpenAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private OpenAction(String name) {
			this.putValue(AbstractAction.NAME, "Open ... ");
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke
					.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_MASK));
			this.putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_O);
			this.putValue(AbstractAction.SHORT_DESCRIPTION,
					"  ... ");
		}

		public void actionPerformed(ActionEvent e) {
			if (MainUI.this.confirm()) {
				MainUI.this.open();
			}
		}

	}

	/**
	 *       
	 */
	private class SaveAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private SaveAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke
					.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_MASK));
			this.putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_S);
			this.putValue(AbstractAction.SHORT_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent e) {
			MainUI.this.save(false);
		}

	}

	/**
	 *       
	 */
	private class SaveAsAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private SaveAsAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SHORT_DESCRIPTION,
					"  ... ");
		}

		public void actionPerformed(ActionEvent e) {
			MainUI.this.save(true);
		}

	}

	/**
	 *    
	 */
	private class AddAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private AddAction(String name) {
			this.putValue(AbstractAction.NAME, "Add graph");
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke
					.getKeyStroke(KeyEvent.VK_A, KeyEvent.CTRL_MASK));
			this.putValue(AbstractAction.MNEMONIC_KEY, KeyEvent.VK_A);
			this.putValue(AbstractAction.SHORT_DESCRIPTION, " ");
		}

		public void actionPerformed(ActionEvent e) {
			MainUI.this.myAddFormulaDialog.setFunction("");
			MainUI.this.myAddFormulaDialog.setProperties(null);
			MainUI.this.addGraph(myAddFormulaDialog.createFunction(true));
		}
	}

	/**
	 *      
	 */
	private class DeleteAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private DeleteAction(String name) {
			this.putValue(AbstractAction.NAME, "Delete graph");
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke
					.getKeyStroke(KeyEvent.VK_DELETE, 0));
			this.putValue(AbstractAction.SHORT_DESCRIPTION, " ");
		}

		public void actionPerformed(ActionEvent e) {
			final JList list = MainUI.this.myFunctionsList;
			final int index = list.getSelectedIndex();
			if (index != -1) {
				DefaultListModel listModel = (DefaultListModel) list.getModel();
				MainUI.this.deleteGraph((IGraph) listModel.getElementAt(index));
			}
		}

	}

	/**
	 *      
	 */
	private class DifferencialAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private DifferencialAction(String name) {
			this.putValue(AbstractAction.NAME, "Differentation");
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.SHORT_DESCRIPTION,
					" ");
		}

		public void actionPerformed(ActionEvent e) {
			IFunction function = MainUI.this.myDifferencialDialog
					.createDifferencial(MainUI.this.myPaintPanel
							.getAbstractView().graphs());
			if (function == null) {
				return;
			}
			MainUI.this.myAddFormulaDialog.setFunction(function.getText());
			MainUI.this.myAddFormulaDialog.setProperties(null);
			MainUI.this.addGraph(MainUI.this.myAddFormulaDialog
					.createFunction(false));
		}
	}

	/**
	 *        
	 */
	private class TangentOrNormalAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private TangentOrNormalAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + "Tangent.png")));
			this.putValue(AbstractAction.SHORT_DESCRIPTION,
					"   ");
		}

		public void actionPerformed(ActionEvent e) {
			String function = MainUI.this.myTangentOrNormalDialog
					.createTangentOrNormal(MainUI.this.myPaintPanel
							.getAbstractView().graphs());
			if (function == null) {
				return;
			}
			MainUI.this.myAddFormulaDialog.setFunction(function);
			MainUI.this.myAddFormulaDialog.setProperties(null);
			MainUI.this.addGraph(MainUI.this.myAddFormulaDialog
					.createFunction(false));
		}
	}

	/**
	 *        
	 */
	private class IntegralAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private IntegralAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.SHORT_DESCRIPTION,
					"  ");
		}

		public void actionPerformed(ActionEvent e) {
			MainUI.this.myIntegralDialog.integrate(MainUI.this.myPaintPanel
					.getAbstractView().graphs());
		}
	}

	private class LeftAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private LeftAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
		}

		public void actionPerformed(ActionEvent e) {
			double oldMinX = MainUI.this.myPaintPanel.getMinimunVisibleX();
			double oldMaxX = MainUI.this.myPaintPanel.getMaximunVisibleX();
			double oldMinY = MainUI.this.myPaintPanel.getMinimunVisibleY();
			double oldMaxY = MainUI.this.myPaintPanel.getMaximunVisibleY();
			double newMinX = oldMinX + MainUI.this.deltaX;
			double newMaxX = oldMaxX + MainUI.this.deltaX;
			MainUI.this.myPaintPanel.setMinimunVisibleX(newMinX);
			MainUI.this.myPaintPanel.setMaximunVisibleX(newMaxX);
			MainUI.this.myPaintPanel.repaint();
			MainUI.this.myUndoStack.push(new ChangePropertiesPaintPanel(
					oldMinX, oldMaxX, oldMinY, oldMaxY, newMinX, newMaxX,
					oldMinY, oldMaxY));
			MainUI.this.myRedoStack = new Stack<CompliteActions>();
			MainUI.this.myRedoButton.setEnabled(false);
			MainUI.this.myUndoButton.setEnabled(true);
			MainUI.this.myDocumentIsSaved = false;
			MainUI.this.mySaveButton.setEnabled(true);
			MainUI.this.mySaveItem.setEnabled(true);
		};

	}
	
	private class RightAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private RightAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
		}

		public void actionPerformed(ActionEvent e) {
			double oldMinX = MainUI.this.myPaintPanel.getMinimunVisibleX();
			double oldMaxX = MainUI.this.myPaintPanel.getMaximunVisibleX();
			double oldMinY = MainUI.this.myPaintPanel.getMinimunVisibleY();
			double oldMaxY = MainUI.this.myPaintPanel.getMaximunVisibleY();
			double newMinX = oldMinX - MainUI.this.deltaX;
			double newMaxX = oldMaxX - MainUI.this.deltaX;
			MainUI.this.myPaintPanel.setMinimunVisibleX(newMinX);
			MainUI.this.myPaintPanel.setMaximunVisibleX(newMaxX);
			MainUI.this.myPaintPanel.repaint();
			MainUI.this.myUndoStack.push(new ChangePropertiesPaintPanel(
					oldMinX, oldMaxX, oldMinY, oldMaxY, newMinX, newMaxX,
					oldMinY, oldMaxY));
			MainUI.this.myRedoStack = new Stack<CompliteActions>();
			MainUI.this.myRedoButton.setEnabled(false);
			MainUI.this.myUndoButton.setEnabled(true);
			MainUI.this.myDocumentIsSaved = false;
			MainUI.this.mySaveButton.setEnabled(true);
			MainUI.this.mySaveItem.setEnabled(true);
		};

	}

	private class UpAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private UpAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
		}

		public void actionPerformed(ActionEvent e) {
			double oldMinX = MainUI.this.myPaintPanel.getMinimunVisibleX();
			double oldMaxX = MainUI.this.myPaintPanel.getMaximunVisibleX();
			double oldMinY = MainUI.this.myPaintPanel.getMinimunVisibleY();
			double oldMaxY = MainUI.this.myPaintPanel.getMaximunVisibleY();
			double newMinY = oldMinY - MainUI.this.deltaY;
			double newMaxY = oldMaxY - MainUI.this.deltaY;
			MainUI.this.myPaintPanel.setMinimunVisibleY(newMinY);
			MainUI.this.myPaintPanel.setMaximunVisibleY(newMaxY);
			MainUI.this.myPaintPanel.repaint();
			MainUI.this.myUndoStack.push(new ChangePropertiesPaintPanel(
					oldMinX, oldMaxX, oldMinY, oldMaxY, oldMinX, oldMaxX,
					newMinY, newMaxY));
			MainUI.this.myRedoStack = new Stack<CompliteActions>();
			MainUI.this.myRedoButton.setEnabled(false);
			MainUI.this.myUndoButton.setEnabled(true);
			MainUI.this.myDocumentIsSaved = false;
			MainUI.this.mySaveButton.setEnabled(true);
			MainUI.this.mySaveItem.setEnabled(true);
		};

	}

	private class DownAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private DownAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
		}

		public void actionPerformed(ActionEvent e) {
			double oldMinX = MainUI.this.myPaintPanel.getMinimunVisibleX();
			double oldMaxX = MainUI.this.myPaintPanel.getMaximunVisibleX();
			double oldMinY = MainUI.this.myPaintPanel.getMinimunVisibleY();
			double oldMaxY = MainUI.this.myPaintPanel.getMaximunVisibleY();
			double newMinY = oldMinY + MainUI.this.deltaY;
			double newMaxY = oldMaxY + MainUI.this.deltaY;
			MainUI.this.myPaintPanel.setMinimunVisibleY(newMinY);
			MainUI.this.myPaintPanel.setMaximunVisibleY(newMaxY);
			MainUI.this.myPaintPanel.repaint();
			MainUI.this.myUndoStack.push(new ChangePropertiesPaintPanel(
					oldMinX, oldMaxX, oldMinY, oldMaxY, oldMinX, oldMaxX,
					newMinY, newMaxY));
			MainUI.this.myRedoStack = new Stack<CompliteActions>();
			MainUI.this.myRedoButton.setEnabled(false);
			MainUI.this.myUndoButton.setEnabled(true);
			MainUI.this.myDocumentIsSaved = false;
			MainUI.this.mySaveButton.setEnabled(true);
			MainUI.this.mySaveItem.setEnabled(true);
		};

	}


	private class ChangeVisibleIntervalAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private ChangeVisibleIntervalAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
		}

		public void actionPerformed(ActionEvent e) {
			if (MainUI.this.myVisibleIntervalDialog.show(
					MainUI.this.myPaintPanel.getMinimunVisibleX(),
					MainUI.this.myPaintPanel.getMaximunVisibleX(),
					MainUI.this.myPaintPanel.getMinimunVisibleY(),
					MainUI.this.myPaintPanel.getMaximunVisibleY())) {
				double oldMinX = MainUI.this.myPaintPanel.getMinimunVisibleX();
				double oldMaxX = MainUI.this.myPaintPanel.getMaximunVisibleX();
				double oldMinY = MainUI.this.myPaintPanel.getMinimunVisibleY();
				double oldMaxY = MainUI.this.myPaintPanel.getMaximunVisibleY();
				MainUI.this.myPaintPanel
						.setMinimunVisibleX(MainUI.this.myVisibleIntervalDialog
								.getMinX());
				MainUI.this.myPaintPanel
						.setMaximunVisibleX(MainUI.this.myVisibleIntervalDialog
								.getMaxX());
				MainUI.this.myPaintPanel
						.setMinimunVisibleY(MainUI.this.myVisibleIntervalDialog
								.getMinY());
				MainUI.this.myPaintPanel
						.setMaximunVisibleY(MainUI.this.myVisibleIntervalDialog
								.getMaxY());
				MainUI.this.myPaintPanel.repaint();
				double newMinX = MainUI.this.myPaintPanel.getMinimunVisibleX();
				double newMaxX = MainUI.this.myPaintPanel.getMaximunVisibleX();
				double newMinY = MainUI.this.myPaintPanel.getMinimunVisibleY();
				double newMaxY = MainUI.this.myPaintPanel.getMaximunVisibleY();
				MainUI.this.myUndoStack.push(new ChangePropertiesPaintPanel(
						oldMinX, oldMaxX, oldMinY, oldMaxY, newMinX, newMaxX,
						newMinY, newMaxY));
				MainUI.this.myRedoStack = new Stack<CompliteActions>();
				MainUI.this.myRedoButton.setEnabled(false);
				MainUI.this.myUndoButton.setEnabled(true);
				MainUI.this.myDocumentIsSaved = false;
				MainUI.this.mySaveButton.setEnabled(true);
				MainUI.this.mySaveItem.setEnabled(true);
			}
		};

	}

	private class UndoAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private UndoAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke
					.getKeyStroke(KeyEvent.CTRL_MASK, KeyEvent.VK_Z));
			this.putValue(AbstractAction.SHORT_DESCRIPTION,
					"  ");
		}

		public void actionPerformed(ActionEvent e) {
			MainUI.this.myRedoStack.push(MainUI.this.myUndoStack.pop()).undo();
			if (MainUI.this.myUndoStack.isEmpty()) {
				MainUI.this.myUndoButton.setEnabled(false);
				MainUI.this.myDocumentIsSaved = true;
				MainUI.this.mySaveButton.setEnabled(false);
				MainUI.this.mySaveItem.setEnabled(false);
			}
			MainUI.this.myRedoButton.setEnabled(true);
		}

	}

	private class RedoAction extends AbstractAction {

		private static final long serialVersionUID = 1L;

		private RedoAction(String name) {
			this.putValue(AbstractAction.NAME, name);
			this.putValue(AbstractAction.SMALL_ICON, new ImageIcon(
					getURL(MainUI.ICON_PATH + name + ".png")));
			this.putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke
					.getKeyStroke(KeyEvent.CTRL_MASK, KeyEvent.VK_Y));
			this.putValue(AbstractAction.SHORT_DESCRIPTION,
					"   ");
		}

		public void actionPerformed(ActionEvent e) {
			MainUI.this.myUndoStack.push(MainUI.this.myRedoStack.pop()).redo();
			if (MainUI.this.myRedoStack.isEmpty()) {
				MainUI.this.myRedoButton.setEnabled(false);
			}
			MainUI.this.myUndoButton.setEnabled(true);
			MainUI.this.myDocumentIsSaved = false;
			MainUI.this.mySaveButton.setEnabled(true);
			MainUI.this.mySaveItem.setEnabled(true);
		}

	}

	private interface CompliteActions {

		public void redo();

		public void undo();

	}

	private class ChangeGraphCompliteAction implements CompliteActions {

		private IGraph myOldGraph;

		private IGraph myNewGraph;

		private ChangeGraphCompliteAction(IGraph oldGraph, IGraph newGraph) {
			this.myOldGraph = oldGraph;
			this.myNewGraph = newGraph;
			MainUI.this.myUndoButton.setEnabled(true);
		}

		public void redo() {
			DefaultListModel listModel = (DefaultListModel) MainUI.this.myFunctionsList
					.getModel();
			int index = listModel.indexOf(this.myOldGraph);
			listModel.removeElement(this.myOldGraph);
			listModel.add(index, this.myNewGraph);
			MainUI.this.myPaintPanel.getAbstractView().changeGraph(
					this.myOldGraph, this.myNewGraph);
		}

		public void undo() {
			DefaultListModel listModel = (DefaultListModel) MainUI.this.myFunctionsList
					.getModel();
			int index = listModel.indexOf(this.myNewGraph);
			listModel.removeElement(this.myNewGraph);
			listModel.add(index, this.myOldGraph);
			MainUI.this.myPaintPanel.getAbstractView().changeGraph(
					this.myNewGraph, this.myOldGraph);
		}

	}

	private class ChangeGraphPaintCompliteAction implements CompliteActions {

		private IGraph myGraph;

		private ChangeGraphPaintCompliteAction(IGraph graph) {
			this.myGraph = graph;
		}

		public void redo() {
			MainUI.this.myPaintPanel.getAbstractView().setGraphPaint(
					this.myGraph, !this.myGraph.isGraphPaint());
		}

		public void undo() {
			MainUI.this.myPaintPanel.getAbstractView().setGraphPaint(
					this.myGraph, !this.myGraph.isGraphPaint());
		}

	}

	private class ChangePropertiesCompliteAction implements CompliteActions {

		private IGraphProperties myOldProperties;

		private IGraphProperties myNewProperties;

		private IGraph myGraph;

		private ChangePropertiesCompliteAction(IGraph graph,
				IGraphProperties oldProperties, IGraphProperties newProperties) {
			this.myGraph = graph;
			this.myOldProperties = oldProperties;
			this.myNewProperties = newProperties;
			MainUI.this.myUndoButton.setEnabled(true);
		}

		public void redo() {
			MainUI.this.myPaintPanel.getAbstractView().setProperties(
					this.myGraph, this.myNewProperties.getMinimumX(),
					this.myNewProperties.getMaximumX(),
					this.myNewProperties.getCountOfSegment(),
					this.myNewProperties.getColor());
		}

		public void undo() {
			MainUI.this.myPaintPanel.getAbstractView().setProperties(
					this.myGraph, this.myOldProperties.getMinimumX(),
					this.myOldProperties.getMaximumX(),
					this.myOldProperties.getCountOfSegment(),
					this.myOldProperties.getColor());
		}

	}

	private class RemoveCompliteAction implements CompliteActions {

		private int myIndex;

		private IGraph myGraph;

		private RemoveCompliteAction(int index, IGraph graph) {
			this.myIndex = index;
			this.myGraph = graph;
		}

		public void redo() {
			DefaultListModel listModel = (DefaultListModel) MainUI.this.myFunctionsList
					.getModel();
			listModel.removeElement(this.myGraph);
			MainUI.this.myPaintPanel.getAbstractView()
					.removeGraph(this.myGraph);
		}

		public void undo() {
			DefaultListModel listModel = (DefaultListModel) MainUI.this.myFunctionsList
					.getModel();
			listModel.add(this.myIndex, this.myGraph);
			MainUI.this.myPaintPanel.getAbstractView().addGraphAt(this.myIndex,
					this.myGraph);
		}

	}

	private class AddCompliteAction implements CompliteActions {

		private IGraph myGraph;

		private AddCompliteAction(IGraph graph) {
			this.myGraph = graph;
		}

		public void redo() {
			DefaultListModel listModel = (DefaultListModel) MainUI.this.myFunctionsList
					.getModel();
			listModel.addElement(this.myGraph);
			MainUI.this.myPaintPanel.getAbstractView().addGraph(this.myGraph);
		}

		public void undo() {
			DefaultListModel listModel = (DefaultListModel) MainUI.this.myFunctionsList
					.getModel();
			listModel.removeElement(this.myGraph);
			MainUI.this.myPaintPanel.getAbstractView()
					.removeGraph(this.myGraph);
		}

	}

	private class ChangePropertiesPaintPanel implements CompliteActions {

		private double myOldMinVisibleX, myNewMinVisibleX;

		private double myOldMaxVisibleX, myNewMaxVisibleX;

		private double myOldMinVisibleY, myNewMinVisibleY;

		private double myOldMaxVisibleY, myNewMaxVisibleY;

		private ChangePropertiesPaintPanel(double oldMinX, double oldMaxX,
				double oldMinY, double oldMaxY, double newMinX, double newMaxX,
				double newMinY, double newMaxY) {
			this.myOldMinVisibleX = oldMinX;
			this.myOldMaxVisibleX = oldMaxX;
			this.myOldMinVisibleY = oldMinY;
			this.myOldMaxVisibleY = oldMaxY;
			this.myNewMinVisibleX = newMinX;
			this.myNewMaxVisibleX = newMaxX;
			this.myNewMinVisibleY = newMinY;
			this.myNewMaxVisibleY = newMaxY;
		}

		public void redo() {
			MainUI.this.myPaintPanel.setMinimunVisibleX(this.myNewMinVisibleX);
			MainUI.this.myPaintPanel.setMaximunVisibleX(this.myNewMaxVisibleX);
			MainUI.this.myPaintPanel.setMinimunVisibleY(this.myNewMinVisibleY);
			MainUI.this.myPaintPanel.setMaximunVisibleY(this.myNewMaxVisibleY);
			MainUI.this.myPaintPanel.repaint();
		}

		public void undo() {
			MainUI.this.myPaintPanel.setMinimunVisibleX(this.myOldMinVisibleX);
			MainUI.this.myPaintPanel.setMaximunVisibleX(this.myOldMaxVisibleX);
			MainUI.this.myPaintPanel.setMinimunVisibleY(this.myOldMinVisibleY);
			MainUI.this.myPaintPanel.setMaximunVisibleY(this.myOldMaxVisibleY);
			MainUI.this.myPaintPanel.repaint();
		}

	}

	private class PaintGraphFileFilter extends FileFilter {

		@Override
		public boolean accept(File f) {
			if (f.isDirectory()) {
				return true;
			}
			return f.getName().endsWith(".pgr");
		}

		@Override
		public String getDescription() {
			return " (*.pgr)";
		}

	}

	private class ImageFileFilter extends FileFilter {

		@Override
		public boolean accept(File f) {
			if (f.isDirectory()) {
				return true;
			}
			return f.getName().endsWith(".png");
		}

		@Override
		public String getDescription() {
			return " PNG (*.png)";
		}

	}

}
