package org.amse.chekh.paint_graph.model.bintree;

import java.util.Iterator;
import java.util.List;

import org.amse.chekh.paint_graph.model.bintree.node.*;
import org.amse.chekh.paint_graph.model.bintree.node.binary.*;
import org.amse.chekh.paint_graph.model.bintree.node.unary.*;
import org.amse.chekh.paint_graph.model.bintree.token.Token;
import org.amse.chekh.paint_graph.model.bintree.token.TokenCreator;

/**
 * ,      
 * 
 * @author 
 * 
 */
public class BinTreeCreator {

	/**
	 *    
	 */
	private List<Token> myTokens;

	/**
	 *    
	 */
	private Iterator<Token> myIterator;

	/**
	 *  
	 */
	private Token myCurrentToken;

	/**
	 *  ,   
	 * 
	 * @param s -
	 *              
	 */
	public BinTreeCreator(String s) {
		this.myTokens = new TokenCreator(s).tokens();
	}

	/**
	 * @return       
	 */
	public AbstractNode createBinTree() {
		this.myIterator = this.myTokens.iterator();
		this.myCurrentToken = this.myIterator.next();
		AbstractNode node = this.add();
		return node;
	}

	/**
	 *    
	 */
	private void nextToken() {
		if (this.myIterator.hasNext()) {
			this.myCurrentToken = this.myIterator.next();
		} else {
			this.myCurrentToken = null;
		}
	}

	/**
	 *      
	 * 
	 * @return  
	 */
	private AbstractNode createThisNode() {
		AbstractNode node;
		if (this.myCurrentToken.isOpenBrace()) {
			this.nextToken(); // after '('
			node = this.add();
			this.nextToken(); // after ')'
		} else {
			if (this.myCurrentToken.isVariable()) {
				node = AbstractNode.VARIABLE_NODE;
				this.nextToken();
			} else if (this.myCurrentToken.isFunction()) {
				node = this.function();
			} else {
				double value = this.myCurrentToken.getNumber();
				if (value == 0.0) {
					node = AbstractNode.ZERO_NODE;
				} else if (value == 1.0) {
					node = AbstractNode.ONE_NODE;
				} else  {
					node = new ConstantNode(value);
				}
				this.nextToken();
			}
		}
		return node;
	}

	/**
	 *      
	 * 
	 * @return  
	 */
	private AbstractNode add() {
		boolean isFirstIteration = true;
		boolean isPlus = true;
		AbstractNode currentNode = null;
		AbstractNode thisNode = null;
		while (this.myCurrentToken != null) {
			if (this.myCurrentToken.isCloseBrace()) {
				break;
			}
			if (!isFirstIteration) {
				if (currentNode == null) {
					if (isPlus) {
						currentNode = thisNode;
					} else {
						currentNode = new MinusNode(new ConstantNode(0.0),
								thisNode);
					}
				} else {
					if (isPlus) {
						currentNode = new PlusNode(currentNode, thisNode);
					} else {
						currentNode = new MinusNode(currentNode, thisNode);
					}
				}
				isPlus = (this.myCurrentToken.isPlus());
				this.nextToken();
			} else { /* isFirstIteration */
				if (this.myCurrentToken.isMinus()) {
					isPlus = false;
					this.nextToken();
				} else {
					isPlus = true;
				}
				isFirstIteration = false;
			}
			thisNode = this.createThisNode();
			if (this.myCurrentToken == null) {
				break;
			}
			if (this.myCurrentToken.isProduct()
					|| this.myCurrentToken.isDivide()) {
				thisNode = this.prod(thisNode);
			} else if (this.myCurrentToken.isPower()) {
				thisNode = new PowerNode(thisNode, this.pow());
				if (this.myCurrentToken == null) {
					break;
				} else {
					if (this.myCurrentToken.isProduct()
							|| this.myCurrentToken.isDivide()) {
						thisNode = this.prod(thisNode);
					}
				}
			}
		}
		if (currentNode == null) {
			if (isPlus) {
				return thisNode;
			} else {
				return new MinusNode(new ConstantNode(0.0), thisNode);
			}
		}
		if (isPlus) {
			return new PlusNode(currentNode, thisNode);
		} else {
			return new MinusNode(currentNode, thisNode);
		}
	}

	/**
	 *      
	 * 
	 * @return  
	 */
	private AbstractNode prod(AbstractNode node) {
		boolean isMulti = this.myCurrentToken.isProduct();
		this.nextToken(); // after '*' or '/'
		AbstractNode currentNode = node;
		AbstractNode thisNode = null;
		while (true) {
			thisNode = this.createThisNode();
			if (this.myCurrentToken == null
					|| this.myCurrentToken.isCloseBrace()
					|| this.myCurrentToken.isPlus()
					|| this.myCurrentToken.isMinus()) {
				break;
			}
			if (this.myCurrentToken.isPower()) {
				thisNode = new PowerNode(thisNode, this.pow());
				if (this.myCurrentToken == null
						|| this.myCurrentToken.isCloseBrace()
						|| this.myCurrentToken.isPlus()
						|| this.myCurrentToken.isMinus()) {
					break;
				}
			}
			if (isMulti) {
				currentNode = new ProductNode(currentNode, thisNode);
			} else {
				currentNode = new DivideNode(currentNode, thisNode);
			}
			isMulti = this.myCurrentToken.isProduct();
			this.nextToken(); // after '*' or '/'
		}
		if (isMulti) {
			return new ProductNode(currentNode, thisNode);
		} else {
			return new DivideNode(currentNode, thisNode);
		}
	}

	/**
	 *    
	 * 
	 * @return  
	 */
	private AbstractNode pow() {
		this.nextToken(); // after '^'
		AbstractNode thisNode = this.createThisNode();
		if (this.myCurrentToken == null) {
			return thisNode;
		}
		if (this.myCurrentToken.isPower()) {
			return new PowerNode(thisNode, this.pow());
		} else {
			return thisNode;
		}
	}

	/**
	 *    
	 * 
	 * @return  
	 */
	private AbstractNode function() {
		String str = this.myCurrentToken.getText();
		this.nextToken(); // after function;
		this.nextToken(); // after '('
		AbstractNode thisNode = this.add();
		this.nextToken(); // after ')'
		if (str.equals("abs")) {
			return new AbsNode(thisNode);
		} else if (str.equals("acos")) {
			return new AcosNode(thisNode);
		} else if (str.equals("asin")) {
			return new AsinNode(thisNode);
		} else if (str.equals("atan")) {
			return new AtanNode(thisNode);
		} else if (str.equals("cos")) {
			return new CosNode(thisNode);
		} else if (str.equals("cosh")) {
			return new CoshNode(thisNode);
		} else if (str.equals("cot")) {
			return new CotNode(thisNode);
		} else if (str.equals("exp")) {
			return new ExpNode(thisNode);
		} else if (str.equals("lg")) {
			return new LgNode(thisNode);
		} else if (str.equals("ln")) {
			return new LnNode(thisNode);
		} else if (str.equals("sin")) {
			return new SinNode(thisNode);
		} else if (str.equals("sinh")) {
			return new SinhNode(thisNode);
		} else if (str.equals("sqrt")) {
			return new SqrtNode(thisNode);
		} else if (str.equals("tan")) {
			return new TanNode(thisNode);
		} else if (str.equals("tanh")) {
			return new TanhNode(thisNode);
		} else {
			return null;
		}
	}

}
