package org.amse.chekh.paint_graph.model.bintree.node.binary;

import org.amse.chekh.paint_graph.model.bintree.node.AbstractNode;
import org.amse.chekh.paint_graph.model.bintree.node.ConstantNode;

/**
 *    
 * 
 * @author 
 */
public class DivideNode extends BinaryFunctionNode {

	public DivideNode(AbstractNode firstArgument, AbstractNode secondArgument) {
		this.myFirstArgument = firstArgument;
		this.mySecondArgument = secondArgument;
	}

	@Override
	public double calculate(double firstArgument, double secondArgument) {
		if (this.canEvaluateInternal(firstArgument, secondArgument)) {
			return (firstArgument / secondArgument);
		} else {
			return Double.NaN;
		}
	}

	@Override
	boolean canEvaluateInternal(double firstArgument, double secondArgument) {
		return secondArgument != 0.0;
	}

	@Override
	public String getFunction() {
		StringBuffer text = new StringBuffer();
		if (this.myFirstArgument instanceof PlusNode
				|| this.myFirstArgument instanceof MinusNode) {
			text.append("(");
			text.append(this.myFirstArgument.getFunction());
			text.append(")");
		} else {
			text.append(this.myFirstArgument.getFunction());
		}
		text.append("/");
		if (this.mySecondArgument instanceof BinaryFunctionNode
				&& !(this.mySecondArgument instanceof PowerNode)) {
			text.append("(");
			text.append(this.mySecondArgument.getFunction());
			text.append(")");
		} else {
			text.append(this.mySecondArgument.getFunction());
		}
		return text.toString();
	}

	@Override
	public AbstractNode getDifferencial() {
		AbstractNode diffFirstArgument = this.myFirstArgument.getDifferencial();
		AbstractNode diffSecondArgument = this.mySecondArgument
				.getDifferencial();
		if (diffSecondArgument == AbstractNode.ZERO_NODE) {
			if (diffFirstArgument == AbstractNode.ZERO_NODE) {
				return AbstractNode.ZERO_NODE;
			} else if (diffFirstArgument == AbstractNode.ONE_NODE) {
				return new DivideNode(AbstractNode.ONE_NODE,
						this.mySecondArgument);
			} else {
				return new DivideNode(diffFirstArgument, this.mySecondArgument);
			}
		}
		AbstractNode first, second;
		if (diffFirstArgument == AbstractNode.ZERO_NODE) {
			first = AbstractNode.ZERO_NODE;
		} else if (diffFirstArgument == AbstractNode.ONE_NODE) {
			first = this.mySecondArgument;
		} else {
			first = new ProductNode(this.mySecondArgument, diffFirstArgument);
		}
		if (diffSecondArgument == AbstractNode.ONE_NODE) {
			second = this.myFirstArgument;
		} else {
			second = new ProductNode(this.myFirstArgument, diffSecondArgument);
		}
		if (first == AbstractNode.ZERO_NODE) {
			return new MinusNode(AbstractNode.ZERO_NODE,
					new DivideNode(second, new PowerNode(this.mySecondArgument,
							new ConstantNode(2.0))));
		} else {
			return new DivideNode(new MinusNode(first, second), new PowerNode(
					this.mySecondArgument, new ConstantNode(2.0)));
		}
	}

}