package org.amse.chekh.paint_graph.model.bintree.node.binary;

import org.amse.chekh.paint_graph.model.bintree.node.AbstractNode;
import org.amse.chekh.paint_graph.model.bintree.node.unary.LnNode;

/**
 *      
 * 
 * @author 
 */
public class PowerNode extends BinaryFunctionNode {

	public PowerNode(AbstractNode firstArgument, AbstractNode secondArgument) {
		this.myFirstArgument = firstArgument;
		this.mySecondArgument = secondArgument;
	}

	@Override
	public double calculate(double firstArgument, double secondArgument) {
		if (this.canEvaluateInternal(firstArgument, secondArgument)) {
			return Math.pow(firstArgument, secondArgument);
		} else {
			return Double.NaN;
		}
	}

	@Override
	boolean canEvaluateInternal(double firstArgument, double secondArgument) {
		if (firstArgument > 0.0) {
			return true;
		} else if (firstArgument < 0.0) {
			return ((int) secondArgument == secondArgument);
		} else { /* firstArgument == 0.0 */
			return (secondArgument > 0);
		}
	}

	@Override
	public String getFunction() {
		StringBuffer text = new StringBuffer();
		if (this.myFirstArgument instanceof BinaryFunctionNode) {
			text.append("(");
			text.append(this.myFirstArgument.getFunction());
			text.append(")");
		} else {
			text.append(this.myFirstArgument.getFunction());
		}
		text.append("^");
		if (this.mySecondArgument instanceof BinaryFunctionNode
				&& !(this.mySecondArgument instanceof PowerNode)) {
			text.append("(");
			text.append(this.mySecondArgument.getFunction());
			text.append(")");
		} else {
			text.append(this.mySecondArgument.getFunction());
		}
		return text.toString();
	}

	@Override
	public AbstractNode getDifferencial() {
		AbstractNode diffFirstArgument = this.myFirstArgument.getDifferencial();
		AbstractNode diffSecondArgument = this.mySecondArgument
				.getDifferencial();
		if (diffFirstArgument == AbstractNode.ZERO_NODE
				&& diffSecondArgument == AbstractNode.ZERO_NODE) {
			return AbstractNode.ZERO_NODE;
		}
		AbstractNode first, second;
		if (diffFirstArgument == AbstractNode.ZERO_NODE) {
			first = AbstractNode.ZERO_NODE;
		} else if (diffFirstArgument == AbstractNode.ONE_NODE) {
			first = new DivideNode(this.mySecondArgument, this.myFirstArgument);
		} else {
			first = new ProductNode(diffFirstArgument, new DivideNode(
					this.mySecondArgument, this.myFirstArgument));
		}
		if (diffSecondArgument == AbstractNode.ZERO_NODE) {
			second = AbstractNode.ZERO_NODE;
		} else if (diffSecondArgument == AbstractNode.ONE_NODE) {
			second = new LnNode(this.myFirstArgument);
		} else {
			second = new ProductNode(diffSecondArgument, new LnNode(
					this.myFirstArgument));
		}
		if (first == AbstractNode.ZERO_NODE) {
			return new ProductNode(this, second);
		} else if (second == AbstractNode.ZERO_NODE) {
			return new ProductNode(this, first);
		} else {
			return new ProductNode(this, new PlusNode(first, second));
		}
	}
}
