package org.amse.chekh.paint_graph.model.bintree.node.unary;

import org.amse.chekh.paint_graph.model.bintree.node.AbstractNode;
import org.amse.chekh.paint_graph.model.bintree.node.ConstantNode;
import org.amse.chekh.paint_graph.model.bintree.node.binary.DivideNode;
import org.amse.chekh.paint_graph.model.bintree.node.binary.MinusNode;
import org.amse.chekh.paint_graph.model.bintree.node.binary.PowerNode;

/**
 *    <b>-</b>
 * 
 * @author 
 */
public class AsinNode extends UnaryFunctionNode {

	public AsinNode(AbstractNode argument) {
		this.myArgument = argument;
	}

	@Override
	public double calculate(double t) {
		if (this.canEvaluateInternal(t)) {
			return Math.asin(t);
		} else {
			return Double.NaN;
		}
	}

	@Override
	boolean canEvaluateInternal(double t) {
		return Math.abs(t) <= 1.0;
	}

	@Override
	public String getFunction() {
		return "asin(" + this.myArgument.getFunction() + ")";
	}

	@Override
	public AbstractNode getDifferencial() {
		AbstractNode diffArgument = this.myArgument.getDifferencial();
		if (diffArgument == AbstractNode.ZERO_NODE) {
			return AbstractNode.ZERO_NODE;
		} else {
			AbstractNode diffNode = new SqrtNode(new MinusNode(
					AbstractNode.ONE_NODE, new PowerNode(this,
							new ConstantNode(2.0))));
			if (diffArgument == AbstractNode.ONE_NODE) {
				return new DivideNode(AbstractNode.ONE_NODE, diffNode);
			} else {
				return new DivideNode(diffArgument, diffNode);
			}
		}
	}

}
