package org.amse.chekh.paint_graph.model.bintree.token;

import java.util.ArrayList;
import java.util.List;

import org.amse.chekh.paint_graph.exception.ParseTreeException;

/**
 *      
 * 
 * @author 
 */
public class TokenCreator {

	/**
	 *   
	 */
	private List<Token> myTokens = new ArrayList<Token>();

	/**
	 * ,    
	 */
	private final String myFormula;

	/**
	 * ,    
	 * 
	 * @param formula -
	 *             
	 */
	public TokenCreator(String formula) {
		formula = formula.toLowerCase();
		this.myFormula = formula.replaceFirst("pi", Math.PI + "");
		this.create();
	}

	/**
	 *   
	 */
	private int countOfOpenBrace = 0;

	/**
	 *   
	 */
	private int countOfCloseBrace = 0;

	/**
	 *  
	 */
	Token lastToken = null;

	/**
	 *   
	 * 
	 * @param i -
	 *                 
	 */
	private void closeBrace(int i) {
		if (countOfCloseBrace == countOfOpenBrace) {
			throw new ParseTreeException(
					"    ", i);
		}
		countOfCloseBrace++;
		if (lastToken.isSignum()) {
			throw new ParseTreeException(
					"      ", i);
		}
		if (lastToken.isOpenBrace()) {
			throw new ParseTreeException(
					"       ",
					i);
		}
		if (lastToken.isFunction()) {
			throw new ParseTreeException(
					"      ,   ",
					i);
		}
		this.myTokens.add(new Token(")", TypeOfToken.BRACE));
	}

	/**
	 *   
	 */
	private void openBrace() {
		countOfOpenBrace++;
		if (lastToken != null) {
			if (lastToken.isVariable() || lastToken.isCloseBrace()
					|| lastToken.isNumber()) {
				this.myTokens.add(new Token("*", TypeOfToken.SIGNUM));
			}
		}
		this.myTokens.add(new Token("(", TypeOfToken.BRACE));
	}

	/**
	 *  
	 * 
	 * @param i -
	 *                
	 * @param c -
	 *             
	 */
	private void signum(int i, char c) {
		if (lastToken != null) {
			if (lastToken.isSignum()) {
				throw new ParseTreeException("     ",
						i);
			}
			if (lastToken.isFunction()) {
				throw new ParseTreeException(
						"      ,   ",
						i);
			}
			if (lastToken.isOpenBrace() && c != '-') {
				throw new ParseTreeException(
						"       "
								+ Character.toString(c), i);
			}
		} else { // lastToken == null
			if (c != '-') {
				throw new ParseTreeException("     "
						+ Character.toString(c), i);
			}
		}
		this.myTokens.add(new Token(Character.toString(c), TypeOfToken.SIGNUM));
	}

	/**
	 *    
	 * 
	 * @param i -
	 *                 
	 * @return (      + 1)
	 */
	private int symbol(int i) {
		char c = this.myFormula.charAt(i);
		if (lastToken != null) {
			if (lastToken.isCloseBrace() || lastToken.isNumber()) {
				this.myTokens.add(new Token("*", TypeOfToken.SIGNUM));
			}
		}
		StringBuffer sb = new StringBuffer();
		do {
			sb.append(c);
			i++;
		} while (i < this.myFormula.length()
				&& isSymbol(c = this.myFormula.charAt(i)));
		this.parseText(sb.toString());
		return i;
	}

	/**
	 *       ()
	 * 
	 * @param i -
	 *                 
	 * @return (      + 1)
	 */
	private int number(int i) {
		char c = this.myFormula.charAt(i);
		boolean pointWas = false;
		if (lastToken != null) {
			if (lastToken.isFunction()) {
				throw new ParseTreeException(
						"      ,   ",
						i);
			}
			if (lastToken.isVariable() || lastToken.isCloseBrace()) {
				this.myTokens.add(new Token("*", TypeOfToken.SIGNUM));
			}
		}
		StringBuffer sb = new StringBuffer();
		do {
			sb.append(c);
			if (c == '.') {
				if (pointWas) {
					throw new ParseTreeException(
							"      1 ! "
									+ sb.toString(), i);
				} else {
					pointWas = true;
				}
			}
			i++;
		} while (i < this.myFormula.length()
				&& (this.isNumber(c = this.myFormula.charAt(i)) || c == '.'));
		this.myTokens.add(new Token(sb.toString(), TypeOfToken.NUMBER));
		return i;
	}

	/**
	 *    
	 * 
	 * @param formula -
	 *             
	 */
	private void create() {
		if (this.myFormula.length() == 0) {
			throw new ParseTreeException(" ", 0);
		}
		for (int i = 0; i < this.myFormula.length();) {
			char c = this.myFormula.charAt(i);
			if (c == ')') {
				this.closeBrace(i);
				i++;
			} else if (c == '(') {
				this.openBrace();
				i++;
			} else if (this.isSignum(c)) {
				this.signum(i, c);
				i++;
			} else if (this.isSymbol(c)) {
				i = this.symbol(i);
			} else if (this.isNumber(c) || c == '.') {
				i = this.number(i);
			} else {
				throw new ParseTreeException("  "
						+ Character.toString(c), i);
			}
			lastToken = this.myTokens.get(this.myTokens.size() - 1);
		}
		if (countOfCloseBrace != countOfOpenBrace) {
			throw new ParseTreeException(
					"    ",
					this.myFormula.length());
		}
		if (lastToken.isSignum()) {
			throw new ParseTreeException(
					"      ",
					this.myFormula.length());
		}
		if (lastToken.isFunction()) {
			throw new ParseTreeException(
					"        ",
					this.myFormula.length());
		}
	}

	/**
	 * ,     <b>str/b>   ,  
	 *  <b>x</b>   <b>function</b>.
	 * 
	 *   <b>str</b>   <b>function</b>  
	 *   ( )    <b>'x'</b>, 
	 *    ,  
	 * 
	 * @return true
	 * 
	 *   <b>str</b>   <b>function</b>   
	 *     <b>'x'</b>,  
	 * @exception ParseTreeException
	 * 
	 *   <b>str</b>    <b>function</b>, 
	 * 
	 * @return false
	 * 
	 * @param str -
	 *             
	 * @param function -
	 *            ,      <b>str</b>
	 */
	private boolean isValid(String str, String function) {
		int count = str.length();
		if (str.endsWith(function)) {
			count -= function.length();
		} else {
			return false;
		}
		for (int i = 0; i < count; i++) {
			if (str.charAt(i) != 'x') {
				return false;
			}
		}
		if (count > 0) {
			this.myTokens.add(new Token("x", TypeOfToken.VARIABLE));
			for (int i = 1; i < count; i++) {
				this.myTokens.add(new Token("*", TypeOfToken.SIGNUM));
				this.myTokens.add(new Token("x", TypeOfToken.VARIABLE));
			}
			if (count < str.length()) {
				this.myTokens.add(new Token("*", TypeOfToken.SIGNUM));
			}
		}
		if (count < str.length()) {
			this.myTokens.add(new Token(function, TypeOfToken.FUNCTION));
		}
		return true;
	}

	/**
	 *    ,   
	 * 
	 * @param text -
	 *             
	 */
	private void parseText(String text) {
		if (isValid(text, "abs")) {
			return;
		} else if (isValid(text, "acos")) {
			return;
		} else if (isValid(text, "asin")) {
			return;
		} else if (isValid(text, "atan")) {
			return;
		} else if (isValid(text, "cos")) {
			return;
		} else if (isValid(text, "cosh")) {
			return;
		} else if (isValid(text, "cot")) {
			return;
		} else if (isValid(text, "exp")) {
			return;
		} else if (isValid(text, "lg")) {
			return;
		} else if (isValid(text, "ln")) {
			return;
		} else if (isValid(text, "sin")) {
			return;
		} else if (isValid(text, "sinh")) {
			return;
		} else if (isValid(text, "sqrt")) {
			return;
		} else if (isValid(text, "tan")) {
			return;
		} else if (isValid(text, "tanh")) {
			return;
		} else if (isValid(text, "")) {
			return;
		} else {
			throw new ParseTreeException("  " + text, 0);
		}
	}

	/**
	 * ,    <b>c/b>    ( )
	 * 
	 * @param c -
	 *             
	 * @return    <b>c/b>    ( )
	 */
	private boolean isNumber(char c) {
		return (c >= '0' && c <= '9');
	}

	/**
	 * ,    <b>c/b> 
	 * 
	 * @param c -
	 *             
	 * @return    <b>c/b> 
	 */
	private boolean isSymbol(char c) {
		return c >= 'a' && c <= 'z';
	}

	/**
	 * ,    <b>c/b> 
	 * 
	 * @param c -
	 *             
	 * @return    <b>c/b> 
	 */
	private boolean isSignum(char c) {
		return c == '+' || c == '-' || c == '*' || c == '/' || c == '^';
	}

	/**
	 * @return    
	 */
	public List<Token> tokens() {
		return this.myTokens;
	}

}
