package org.amse.chekh.paint_graph.tests.function_tests;

import junit.framework.TestCase;

import org.amse.chekh.paint_graph.model.IFunction;
import org.amse.chekh.paint_graph.model.impl.FunctionFactory;

public class CanEvaluateTests extends TestCase {

	public void testConst() {
		IFunction f = FunctionFactory.createFunction("0.0");
		assertTrue(f.canEvaluate(0.0));
	}

	public void testVariable() {
		IFunction f = FunctionFactory.createFunction("x");
		assertTrue(f.canEvaluate(1.0));
	}

	public void testPlus() {
		IFunction f = FunctionFactory.createFunction("x+x");
		assertTrue(f.canEvaluate(-1.34));
	}

	public void testMinus() {
		IFunction f = FunctionFactory.createFunction("x-x");
		assertTrue(f.canEvaluate(-12.5));
	}
	
	public void testProduct() {
		IFunction f = FunctionFactory.createFunction("x*x");
		assertTrue(f.canEvaluate(-10.5));
	}

	public void testDivide1() {
		IFunction f = FunctionFactory.createFunction("x/x");
		assertTrue(f.canEvaluate(1.23));
	}

	public void testDivide2() {
		IFunction f = FunctionFactory.createFunction("x/x");
		assertFalse(f.canEvaluate(0.0));
	}

	public void testAbs1() {
		IFunction f = FunctionFactory.createFunction("abs(x)");
		assertTrue(f.canEvaluate(1.23));
	}

	public void testAbs2() {
		IFunction f = FunctionFactory.createFunction("abs(x)");
		assertTrue(f.canEvaluate(-1.123));
	}

	public void testAcos1() {
		IFunction f = FunctionFactory.createFunction("acos(x)");
		assertTrue(f.canEvaluate(0.35));
	}

	public void testAcos2() {
		IFunction f = FunctionFactory.createFunction("acos(x)");
		assertFalse(f.canEvaluate(-10.0));
	}
	
	public void testAcos3() {
		IFunction f = FunctionFactory.createFunction("acos(x)");
		assertTrue(f.canEvaluate(-1.0));
	}
	
	public void testAcos4() {
		IFunction f = FunctionFactory.createFunction("acos(x)");
		assertTrue(f.canEvaluate(1.0));
	}

	public void testAsin1() {
		IFunction f = FunctionFactory.createFunction("asin(x)");
		assertTrue(f.canEvaluate(0.35));
	}

	public void testAsin2() {
		IFunction f = FunctionFactory.createFunction("asin(x)");
		assertFalse(f.canEvaluate(-10.0));
	}
	
	public void testAsin3() {
		IFunction f = FunctionFactory.createFunction("asin(x)");
		assertTrue(f.canEvaluate(-1.0));
	}
	
	public void testAsin4() {
		IFunction f = FunctionFactory.createFunction("asin(x)");
		assertTrue(f.canEvaluate(1.0));
	}
	
	public void testAtan1() {
		IFunction f = FunctionFactory.createFunction("atan(x)");
		assertTrue(f.canEvaluate(191.24));
	}
	
	public void testAtan2() {
		IFunction f = FunctionFactory.createFunction("atan(x)");
		assertTrue(f.canEvaluate(-191.24));
	}

	public void testCos1() {
		IFunction f = FunctionFactory.createFunction("cos(x)");
		assertTrue(f.canEvaluate(0.75));
	}
	
	public void testCos2() {
		IFunction f = FunctionFactory.createFunction("cos(x)");
		assertTrue(f.canEvaluate(-3.75));
	}

	public void testCosh1() {
		IFunction f = FunctionFactory.createFunction("cosh(x)");
		assertTrue(f.canEvaluate(450.0));
	}
	
	public void testCosh2() {
		IFunction f = FunctionFactory.createFunction("cosh(x)");
		assertTrue(f.canEvaluate(-450.0));
	}

	public void testCot1() {
		IFunction f = FunctionFactory.createFunction("cot(x)");
		assertTrue(f.canEvaluate(7.605));
	}
	
	public void testCot2() {
		IFunction f = FunctionFactory.createFunction("cot(x)");
		assertTrue(f.canEvaluate(-7.605));
	}

	public void testCot3() {
		IFunction f = FunctionFactory.createFunction("cot(x)");
		assertFalse(f.canEvaluate(-Math.PI * 6));
	}
	
	public void testCot4() {
		IFunction f = FunctionFactory.createFunction("cot(x)");
		assertFalse(f.canEvaluate(Math.PI * 8));
	}

	public void testExp1() {
		IFunction f = FunctionFactory.createFunction("exp(x)");
		assertTrue(f.canEvaluate(20.0));
	}
	
	public void testExp2() {
		IFunction f = FunctionFactory.createFunction("exp(x)");
		assertTrue(f.canEvaluate(-20.0));
	}

	public void testLg1() {
		IFunction f = FunctionFactory.createFunction("lg(x)");
		assertTrue(f.canEvaluate(0.11));
	}

	public void testLg2() {
		IFunction f = FunctionFactory.createFunction("lg(x)");
		assertFalse(f.canEvaluate(-4.1));
	}
	
	public void testLg3() {
		IFunction f = FunctionFactory.createFunction("lg(x)");
		assertFalse(f.canEvaluate(0.0));
	}

	public void testLn1() {
		IFunction f = FunctionFactory.createFunction("ln(x)");
		assertTrue(f.canEvaluate(2.1));
	}

	public void testLn2() {
		IFunction f = FunctionFactory.createFunction("ln(x)");
		assertFalse(f.canEvaluate(-30.12));
	}
	
	public void testLn3() {
		IFunction f = FunctionFactory.createFunction("ln(x)");
		assertFalse(f.canEvaluate(0.0));
	}

	public void testSin1() {
		IFunction f = FunctionFactory.createFunction("sin(x)");
		assertTrue(f.canEvaluate(10.32));
	}
	
	public void testSin2() {
		IFunction f = FunctionFactory.createFunction("sin(x)");
		assertTrue(f.canEvaluate(-10.32));
	}

	public void testSinh1() {
		IFunction f = FunctionFactory.createFunction("sinh(x)");
		assertTrue(f.canEvaluate(4.1));
	}
	
	public void testSinh2() {
		IFunction f = FunctionFactory.createFunction("sinh(x)");
		assertTrue(f.canEvaluate(-4.1));
	}

	public void testSqrt1() {
		IFunction f = FunctionFactory.createFunction("sqrt(x)");
		assertTrue(f.canEvaluate(123.45));
	}
	
	public void testSqrt2() {
		IFunction f = FunctionFactory.createFunction("sqrt(x)");
		assertTrue(f.canEvaluate(0.0));
	}

	public void testSqrt3() {
		IFunction f = FunctionFactory.createFunction("sqrt(x)");
		assertFalse(f.canEvaluate(-123.45));
	}

	public void testTan1() {
		IFunction f = FunctionFactory.createFunction("tan(x)");
		assertTrue(f.canEvaluate(63.425));
	}
	
	public void testTan2() {
		IFunction f = FunctionFactory.createFunction("tan(x)");
		assertTrue(f.canEvaluate(-63.425));
	}

	public void testTan3() {
		IFunction f = FunctionFactory.createFunction("tan(x)");
		assertFalse(f.canEvaluate(7 * Math.PI / 2));
	}
	
	public void testTanh1() {
		IFunction f = FunctionFactory.createFunction("tanh(x)");
		assertTrue(f.canEvaluate(-53.12));
	}
	
	public void testTanh2() {
		IFunction f = FunctionFactory.createFunction("tanh(x)");
		assertTrue(f.canEvaluate(53.12));
	}

	public void testPower1() {
		IFunction f = FunctionFactory.createFunction("2^x");
		assertTrue(f.canEvaluate(2.0));
	}
	
	public void testPower2() {
		IFunction f = FunctionFactory.createFunction("2^x");
		assertTrue(f.canEvaluate(-2.0));
	}
	
	public void testPower3() {
		IFunction f = FunctionFactory.createFunction("2^x");
		assertTrue(f.canEvaluate(0.0));
	}
	
	public void testPower4() {
		IFunction f = FunctionFactory.createFunction("2^x");
		assertTrue(f.canEvaluate(2.2));
	}
	
	public void testPower5() {
		IFunction f = FunctionFactory.createFunction("2^x");
		assertTrue(f.canEvaluate(-2.2));
	}
	
	public void testPower6() {
		IFunction f = FunctionFactory.createFunction("2.2^x");
		assertTrue(f.canEvaluate(2.0));
	}

	public void testPower7() {
		IFunction f = FunctionFactory.createFunction("2.2^x");
		assertTrue(f.canEvaluate(-2.0));
	}
	
	public void testPower8() {
		IFunction f = FunctionFactory.createFunction("2.2^x");
		assertTrue(f.canEvaluate(0.0));
	}
	
	public void testPower9() {
		IFunction f = FunctionFactory.createFunction("2.2^x");
		assertTrue(f.canEvaluate(2.2));
	}
	
	public void testPower10() {
		IFunction f = FunctionFactory.createFunction("2.2^x");
		assertTrue(f.canEvaluate(-2.2));
	}
	
	public void testPower11() {
		IFunction f = FunctionFactory.createFunction("0.0^x");
		assertTrue(f.canEvaluate(2.0));
	}
	
	public void testPower12() {
		IFunction f = FunctionFactory.createFunction("0.0^x");
		assertFalse(f.canEvaluate(-2.0));
	}
	
	public void testPower13() {
		IFunction f = FunctionFactory.createFunction("0.0^x");
		assertFalse(f.canEvaluate(0.0));
	}
	
	public void testPower14() {
		IFunction f = FunctionFactory.createFunction("0.0^x");
		assertTrue(f.canEvaluate(2.2));
	}
	
	public void testPower15() {
		IFunction f = FunctionFactory.createFunction("0.0^x");
		assertFalse(f.canEvaluate(-2.2));
	}
	
	public void testPower16() {
		IFunction f = FunctionFactory.createFunction("(-2.0)^x");
		assertTrue(f.canEvaluate(2.0));
	}

	public void testPower17() {
		IFunction f = FunctionFactory.createFunction("(-2.0)^x");
		assertTrue(f.canEvaluate(-2.0));
	}
	
	public void testPower18() {
		IFunction f = FunctionFactory.createFunction("(-2.0)^x");
		assertTrue(f.canEvaluate(0.0));
	}
	
	public void testPower19() {
		IFunction f = FunctionFactory.createFunction("(-2.0)^x");
		assertFalse(f.canEvaluate(2.2));
	}
	
	public void testPower20() {
		IFunction f = FunctionFactory.createFunction("(-2.0)^x");
		assertFalse(f.canEvaluate(-2.2));
	}
	
	public void testPower21() {
		IFunction f = FunctionFactory.createFunction("-2.0^x");
		assertTrue(f.canEvaluate(2.0));
	}
	
	public void testPower22() {
		IFunction f = FunctionFactory.createFunction("-2.0^x");
		assertTrue(f.canEvaluate(-2.0));
	}
	
	public void testPower23() {
		IFunction f = FunctionFactory.createFunction("-2.0^x");
		assertTrue(f.canEvaluate(0.0));
	}
	
	public void testPower24() {
		IFunction f = FunctionFactory.createFunction("-2.0^x");
		assertTrue(f.canEvaluate(2.2));
	}
	
	public void testPower25() {
		IFunction f = FunctionFactory.createFunction("-2.0^x");
		assertTrue(f.canEvaluate(-2.2));
	}
	
	public void testPower26() {
		IFunction f = FunctionFactory.createFunction("(-2.2)^x");
		assertTrue(f.canEvaluate(2.0));
	}

	public void testPower27() {
		IFunction f = FunctionFactory.createFunction("(-2.2)^x");
		assertTrue(f.canEvaluate(-2.0));
	}
	
	public void testPower28() {
		IFunction f = FunctionFactory.createFunction("(-2.2)^x");
		assertTrue(f.canEvaluate(0.0));
	}
	
	public void testPower29() {
		IFunction f = FunctionFactory.createFunction("(-2.2)^x");
		assertFalse(f.canEvaluate(2.2));
	}
	
	public void testPower30() {
		IFunction f = FunctionFactory.createFunction("(-2.2)^x");
		assertFalse(f.canEvaluate(-2.2));
	}
		
	public void test42() {
		IFunction f = FunctionFactory.createFunction("sin(x/x)");
		assertFalse(f.canEvaluate(0.0));
	}
	
	public void test43() {
		IFunction f = FunctionFactory.createFunction("abs(lg(x))");
		assertFalse(f.canEvaluate(-5.0));
	}
	
	public void test44() {
		IFunction f = FunctionFactory.createFunction("exp(abs(acos(x)))");
		assertFalse(f.canEvaluate(5.0));
	}
	
}
