package org.amse.chekh.paint_graph.tests.function_tests;

import junit.framework.TestCase;

import org.amse.chekh.paint_graph.model.IFunction;
import org.amse.chekh.paint_graph.model.impl.FunctionFactory;

public class EvaluateTests extends TestCase {

	public void test1() {
		IFunction f = FunctionFactory.createFunction("0.0");
		assertEquals(0.0, f.evaluate(0.0));
	}

	public void test2() {
		IFunction f = FunctionFactory.createFunction("123.456");
		assertEquals(123.456, f.evaluate(-53.12));
	}

	public void test3() {
		IFunction f = FunctionFactory.createFunction("x");
		assertEquals(153.198, f.evaluate(153.198));
	}

	public void test4() {
		IFunction f = FunctionFactory.createFunction("x+x");
		assertEquals(-246.4, f.evaluate(-123.2));
	}

	public void test5() {
		IFunction f = FunctionFactory.createFunction("x*x");
		assertEquals(23.2 * 23.2, f.evaluate(-23.2));
	}

	public void test6() {
		IFunction f = FunctionFactory.createFunction("x/x");
		assertEquals(1.0, f.evaluate(-33.2));
	}

	public void test7() {
		IFunction f = FunctionFactory.createFunction("abs(x)");
		assertEquals(246.23, f.evaluate(246.23));
	}

	public void test8() {
		IFunction f = FunctionFactory.createFunction("abs(x)");
		assertEquals(246.23, f.evaluate(-246.23));
	}

	public void test9() {
		IFunction f = FunctionFactory.createFunction("acos(x)");
		assertEquals(Math.acos(0.123), f.evaluate(0.123));
	}

	public void test10() {
		IFunction f = FunctionFactory.createFunction("asin(x)");
		assertEquals(Math.asin(-0.123), f.evaluate(-0.123));
	}

	public void test11() {
		IFunction f = FunctionFactory.createFunction("atan(x)");
		assertEquals(Math.atan(-13.13), f.evaluate(-13.13));
	}

	public void test12() {
		IFunction f = FunctionFactory.createFunction("cos(x)");
		assertEquals(Math.cos(94.53), f.evaluate(94.53));
	}

	public void test13() {
		IFunction f = FunctionFactory.createFunction("cosh(x)");
		assertEquals(Math.cosh(-120.25), f.evaluate(-120.25));
	}

	public void test14() {
		IFunction f = FunctionFactory.createFunction("cot(x)");
		assertEquals(1 / Math.tan(0.121), f.evaluate(0.121));
	}

	public void test15() {
		IFunction f = FunctionFactory.createFunction("exp(x)");
		assertEquals(Math.exp(12.52), f.evaluate(12.52));
	}

	public void test16() {
		IFunction f = FunctionFactory.createFunction("exp(x)");
		assertEquals(Math.exp(-5.1), f.evaluate(-5.1));
	}

	public void test17() {
		IFunction f = FunctionFactory.createFunction("lg(x)");
		assertEquals(Math.log10(1.12), f.evaluate(1.12));
	}

	public void test18() {
		IFunction f = FunctionFactory.createFunction("ln(x)");
		assertEquals(Math.log(2.35), f.evaluate(2.35));
	}

	public void test19() {
		IFunction f = FunctionFactory.createFunction("sin(x)");
		assertEquals(Math.sin(25.52), f.evaluate(25.52));
	}

	public void test20() {
		IFunction f = FunctionFactory.createFunction("sinh(x)");
		assertEquals(Math.sinh(31.31), f.evaluate(31.31));
	}

	public void test21() {
		IFunction f = FunctionFactory.createFunction("sqrt(x)");
		assertEquals(Math.sqrt(42.21), f.evaluate(42.21));
	}

	public void test22() {
		IFunction f = FunctionFactory.createFunction("sinh(x)");
		assertEquals(Math.sinh(-13.45), f.evaluate(-13.45));
	}

	public void test23() {
		IFunction f = FunctionFactory.createFunction("tan(x)");
		assertEquals(Math.tan(53.12), f.evaluate(53.12));
	}

	public void test24() {
		IFunction f = FunctionFactory.createFunction("tanh(x)");
		assertEquals(Math.tanh(-231.221), f.evaluate(-231.221));
	}

	public void test25() {
		IFunction f = FunctionFactory
				.createFunction("sin(x)*cos(x)/exp(x)+x^2");
		assertEquals(Math.sin(12.5) * Math.cos(12.5) / Math.exp(12.5) + 12.5
				* 12.5, f.evaluate(12.5));
	}

	public void test26() {
		IFunction f = FunctionFactory.createFunction("1+(2+2)");
		assertEquals(5.0, f.evaluate(12.5));
	}

	public void test27() {
		IFunction f = FunctionFactory.createFunction("1-(2+2)");
		assertEquals(-3.0, f.evaluate(12.5));
	}

	public void test28() {
		IFunction f = FunctionFactory.createFunction("1+2+3");
		assertEquals(6.0, f.evaluate(12.5));
	}

	public void test29() {
		IFunction f = FunctionFactory.createFunction("(1+2)+3");
		assertEquals(6.0, f.evaluate(12.5));
	}

	public void test30() {
		IFunction f = FunctionFactory.createFunction("1+2-3");
		assertEquals(0.0, f.evaluate(12.5));
	}

	public void test31() {
		IFunction f = FunctionFactory.createFunction("(1+2)-3");
		assertEquals(0.0, f.evaluate(12.5));
	}

	public void test32() {
		IFunction f = FunctionFactory.createFunction("1+2*3");
		assertEquals(7.0, f.evaluate(12.5));
	}

	public void test33() {
		IFunction f = FunctionFactory.createFunction("(1+2)*3");
		assertEquals(9.0, f.evaluate(12.5));
	}

	public void test34() {
		IFunction f = FunctionFactory.createFunction("1+2/3");
		assertEquals(1 + 2.0 / 3.0, f.evaluate(12.5));
	}

	public void test35() {
		IFunction f = FunctionFactory.createFunction("(1+2)/3");
		assertEquals(1.0, f.evaluate(12.5));
	}

	public void test36() {
		IFunction f = FunctionFactory.createFunction("1+2^3");
		assertEquals(9.0, f.evaluate(12.5));
	}

	public void test37() {
		IFunction f = FunctionFactory.createFunction("(1+2)^3");
		assertEquals(27.0, f.evaluate(12.5));
	}

	public void test38() {
		IFunction f = FunctionFactory.createFunction("2*(2+3)");
		assertEquals(10.0, f.evaluate(12.5));
	}

	public void test39() {
		IFunction f = FunctionFactory.createFunction("2/(2+3)");
		assertEquals(0.4, f.evaluate(12.5));
	}

	public void test40() {
		IFunction f = FunctionFactory.createFunction("2*3+2");
		assertEquals(8.0, f.evaluate(12.5));
	}

	public void test41() {
		IFunction f = FunctionFactory.createFunction("(2*3)+2");
		assertEquals(8.0, f.evaluate(12.5));
	}

	public void test42() {
		IFunction f = FunctionFactory.createFunction("2*3-2");
		assertEquals(4.0, f.evaluate(12.5));
	}

	public void test43() {
		IFunction f = FunctionFactory.createFunction("(2*3)-2");
		assertEquals(4.0, f.evaluate(12.5));
	}

	public void test44() {
		IFunction f = FunctionFactory.createFunction("2*3*3");
		assertEquals(18.0, f.evaluate(12.5));
	}

	public void test45() {
		IFunction f = FunctionFactory.createFunction("(2*3)*3");
		assertEquals(18.0, f.evaluate(12.5));
	}

	public void test46() {
		IFunction f = FunctionFactory.createFunction("2*3/2");
		assertEquals(3.0, f.evaluate(12.5));
	}

	public void test47() {
		IFunction f = FunctionFactory.createFunction("(2*3)/2");
		assertEquals(3.0, f.evaluate(12.5));
	}

	public void test48() {
		IFunction f = FunctionFactory.createFunction("2*3^3");
		assertEquals(54.0, f.evaluate(12.5));
	}

	public void test49() {
		IFunction f = FunctionFactory.createFunction("(2*3)^3");
		assertEquals(216.0, f.evaluate(12.5));
	}

	public void test50() {
		IFunction f = FunctionFactory.createFunction("2^(3+4)");
		assertEquals(128.0, f.evaluate(12.5));
	}

	public void test51() {
		IFunction f = FunctionFactory.createFunction("3^2+3");
		assertEquals(12.0, f.evaluate(12.5));
	}

	public void test52() {
		IFunction f = FunctionFactory.createFunction("3^2-3");
		assertEquals(6.0, f.evaluate(12.5));
	}

	public void test53() {
		IFunction f = FunctionFactory.createFunction("3^2*3");
		assertEquals(27.0, f.evaluate(12.5));
	}

	public void test54() {
		IFunction f = FunctionFactory.createFunction("3^2/6");
		assertEquals(1.50, f.evaluate(12.5));
	}

	public void test55() {
		IFunction f = FunctionFactory.createFunction("3^2^3");
		assertEquals(6561.0, f.evaluate(12.5));
	}

	public void test56() {
		IFunction f = FunctionFactory.createFunction("(3^2)^3");
		assertEquals(729.0, f.evaluate(12.5));
	}

	public void test57() {
		IFunction f = FunctionFactory.createFunction("-1-2+4");
		assertEquals(1.0, f.evaluate(12.5));
	}

	public void test58() {
		IFunction f = FunctionFactory.createFunction("(-1-2)+4");
		assertEquals(1.0, f.evaluate(12.5));
	}

	public void test59() {
		IFunction f = FunctionFactory.createFunction("-1-2-3");
		assertEquals(-6.0, f.evaluate(12.5));
	}

	public void test60() {
		IFunction f = FunctionFactory.createFunction("(-1-2)-3");
		assertEquals(-6.0, f.evaluate(12.5));
	}

	public void test61() {
		IFunction f = FunctionFactory.createFunction("-1-2*3");
		assertEquals(-7.0, f.evaluate(12.5));
	}

	public void test62() {
		IFunction f = FunctionFactory.createFunction("(-1-2)*3");
		assertEquals(-9.0, f.evaluate(12.5));
	}

	public void test63() {
		IFunction f = FunctionFactory.createFunction("-1-2/3");
		assertEquals(-1 - 2.0 / 3.0, f.evaluate(12.5));
	}

	public void test64() {
		IFunction f = FunctionFactory.createFunction("(-1-2)/3");
		assertEquals(-1.0, f.evaluate(12.5));
	}

	public void test65() {
		IFunction f = FunctionFactory.createFunction("-1-2^3");
		assertEquals(-9.0, f.evaluate(12.5));
	}

	public void test66() {
		IFunction f = FunctionFactory.createFunction("(-1-2)^3");
		assertEquals(-27.0, f.evaluate(12.5));
	}

	public void test67() {
		IFunction f = FunctionFactory.createFunction("2/3+2");
		assertEquals(2.0 / 3.0 + 2.0, f.evaluate(12.5));
	}

	public void test68() {
		IFunction f = FunctionFactory.createFunction("(2/3)+2");
		assertEquals(2.0 + 2.0 / 3.0, f.evaluate(12.5));
	}

	public void test69() {
		IFunction f = FunctionFactory.createFunction("2/3-2");
		assertEquals(2.0 / 3.0 - 2.0, f.evaluate(12.5));
	}

	public void test70() {
		IFunction f = FunctionFactory.createFunction("(2/3)-2");
		assertEquals(2.0 / 3.0 - 2.0, f.evaluate(12.5));
	}

	public void test71() {
		IFunction f = FunctionFactory.createFunction("2/3*3");
		assertEquals(2.0, f.evaluate(12.5));
	}

	public void test72() {
		IFunction f = FunctionFactory.createFunction("(2/3)*3");
		assertEquals(2.0, f.evaluate(12.5));
	}

	public void test73() {
		IFunction f = FunctionFactory.createFunction("2/3/2");
		assertEquals(1.0 / 3.0, f.evaluate(12.5));
	}

	public void test74() {
		IFunction f = FunctionFactory.createFunction("(2/3)/2");
		assertEquals(1.0 / 3.0, f.evaluate(12.5));
	}

	public void test75() {
		IFunction f = FunctionFactory.createFunction("2/3^3");
		assertEquals(2.0 / 27.0, f.evaluate(12.5));
	}

	public void test76() {
		IFunction f = FunctionFactory.createFunction("(2/3)^3");
		assertEquals(Math.pow(2.0 / 3.0, 3), f.evaluate(12.5));
	}

	public void test77() {
		IFunction f = FunctionFactory.createFunction("x2.1");
		assertEquals(2.1 * 12.5, f.evaluate(12.5));
	}

	public void test78() {
		IFunction f = FunctionFactory.createFunction("2.2x5.5");
		assertEquals(2.2*5.5*12.5, f.evaluate(12.5));
	}

	public void test79() {
		IFunction f = FunctionFactory.createFunction("xx2");
		assertEquals(12.5 * 12.5 * 2, f.evaluate(12.5));
	}

	public void test80() {
		IFunction f = FunctionFactory.createFunction("(-5)(-x)");
		assertEquals(5 * 12.5, f.evaluate(12.5));
	}

	public void test81() {
		IFunction f = FunctionFactory.createFunction("x(-5)");
		assertEquals(-5 * 12.5, f.evaluate(12.5));
	}

	public void test82() {
		IFunction f = FunctionFactory.createFunction("5(x+x)");
		assertEquals(125.0, f.evaluate(12.5));
	}

	public void test83() {
		IFunction f = FunctionFactory.createFunction("(-x)5");
		assertEquals(-5 * 12.5, f.evaluate(12.5));
	}

	public void test84() {
		IFunction f = FunctionFactory.createFunction("xsin(x)");
		assertEquals(12.5 * Math.sin(12.5), f.evaluate(12.5));
	}

	public void test85() {
		IFunction f = FunctionFactory.createFunction("5cos(x)");
		assertEquals(5 * Math.cos(12.5), f.evaluate(12.5));
	}

	public void test86() {
		IFunction f = FunctionFactory.createFunction("(-5)tan(x)");
		assertEquals(-5 * Math.tan(12.5), f.evaluate(12.5));
	}

	public void test87() {
		IFunction f = FunctionFactory.createFunction("xx/x*xx^x");
		assertEquals(16.0, f.evaluate(2.0));
	}
	
	public void test88() {
		IFunction f = FunctionFactory.createFunction("x2.1x");
		assertEquals(11.109, f.evaluate(2.3));
	}

}
