package org.amse.chekh.paint_graph.tests.reader_tests;

import java.awt.Color;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import org.amse.chekh.paint_graph.abstact_view.IAbstractView;
import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.amse.chekh.paint_graph.abstact_view.impl.AbstractViewFactory;
import org.amse.chekh.paint_graph.abstact_view.impl.GraphFactory;
import org.amse.chekh.paint_graph.io.reader.Reader;
import org.amse.chekh.paint_graph.model.impl.FunctionFactory;
import org.xml.sax.SAXException;

import junit.framework.TestCase;

public class ReaderTests extends TestCase {

	public void test1() {
		IAbstractView correctView = AbstractViewFactory.createAbsractView();
		IGraph g = GraphFactory.createGraph(
				FunctionFactory.createFunction("x"), -10, 10, 1000,
				Color.BLACK, true);
		correctView.addGraph(g);
		try {
			IAbstractView readView = AbstractViewFactory.createAbsractView();
			Reader r = new Reader(System.getProperty("user.dir")
					+ File.separatorChar + "org" + File.separatorChar + "amse"
					+ File.separatorChar + "chekh" + File.separatorChar
					+ "paint_graph" + File.separatorChar + "tests"
					+ File.separatorChar + "reader_tests" + File.separatorChar
					+ "1.pgr");
			readView.addAllGraphs(r.read());
			List<IGraph> readList = readView.graphs();
			List<IGraph> correctList = correctView.graphs();
			assertEquals("     X", -10.0, r.getMinimunVisibleX());
			assertEquals("     X", 10.0, r.getMaximunVisibleX());
			assertEquals("     Y", -10.0, r.getMinimunVisibleY());
			assertEquals("     Y", 10.0, r.getMaximunVisibleY());
			assertEquals("   !",
					correctList.size(), readList.size());
			Iterator<IGraph> it = correctList.iterator();
			int i = 0;
			for (IGraph readGraph : readList) {
				IGraph correctGraph = it.next();
				assertEquals("     " + i,
						correctGraph.getFunction().getText(), readGraph
								.getFunction().getText());
				assertEquals(
						"   'minimum_X'   "
								+ i,
						correctGraph.getProperties().getMinimumX(), readGraph
								.getProperties().getMinimumX());
				assertEquals(
						"   'maximum_X'   "
								+ i,
						correctGraph.getProperties().getMaximumX(), readGraph
								.getProperties().getMaximumX());
				assertEquals(
						"   'count_of_step'   "
								+ i, correctGraph.getProperties()
								.getCountOfSegment(), readGraph.getProperties()
								.getCountOfSegment());
				assertEquals(
						"   'color'   "
								+ i, correctGraph.getProperties().getColor(),
						readGraph.getProperties().getColor());
				assertEquals(
						"   'isPaint'   "
								+ i, correctGraph.isGraphPaint(), readGraph
								.isGraphPaint());
				i++;
			}
		} catch (IOException ex) {
		} catch (ParserConfigurationException ex) {
		} catch (SAXException ex) {
		}
	}

	public void test2() {
		IAbstractView correctView = AbstractViewFactory.createAbsractView();
		IGraph g = GraphFactory.createGraph(FunctionFactory
				.createFunction("x*x"), -5.0, 6.0, 100, Color.RED, true);
		correctView.addGraph(g);
		g = GraphFactory.createGraph(FunctionFactory.createFunction("1/x"),
				-1.0, 1.0, 20000, Color.BLACK, true);
		correctView.addGraph(g);
		g = GraphFactory.createGraph(FunctionFactory
				.createFunction("-sqrt(abs(x))"), -100.25, 200.25, 1000,
				Color.BLUE, true);
		correctView.addGraph(g);
		g = GraphFactory.createGraph(FunctionFactory.createFunction("4"),
				-10.0, 10.0, 5, Color.YELLOW, false);
		correctView.addGraph(g);
		g = GraphFactory.createGraph(FunctionFactory.createFunction("f"),
				-12.34, 12.34, 1234, Color.GREEN, false);
		correctView.addGraph(g);
		try {
			IAbstractView readView = AbstractViewFactory.createAbsractView();
			Reader r = new Reader(System.getProperty("user.dir")
					+ File.separatorChar + "org" + File.separatorChar + "amse"
					+ File.separatorChar + "chekh" + File.separatorChar
					+ "paint_graph" + File.separatorChar + "tests"
					+ File.separatorChar + "reader_tests" + File.separatorChar
					+ "2.pgr");
			readView.addAllGraphs(r.read());
			List<IGraph> readList = readView.graphs();
			List<IGraph> correctList = correctView.graphs();
			assertEquals("     X", 20.0, r.getMinimunVisibleX());
			assertEquals("     X", 102.01, r.getMaximunVisibleX());
			assertEquals("     Y", -10.5, r.getMinimunVisibleY());
			assertEquals("     Y", -5.5, r.getMaximunVisibleY());
			assertEquals("   !",
					correctList.size(), readList.size());
			Iterator<IGraph> it = correctList.iterator();
			int i = 0;
			for (IGraph readGraph : readList) {
				IGraph correctGraph = it.next();
				assertEquals("     " + i,
						correctGraph.getFunction().getText(), readGraph
								.getFunction().getText());
				assertEquals(
						"   'minimum_X'   "
								+ i,
						correctGraph.getProperties().getMinimumX(), readGraph
								.getProperties().getMinimumX());
				assertEquals(
						"   'maximum_X'   "
								+ i,
						correctGraph.getProperties().getMaximumX(), readGraph
								.getProperties().getMaximumX());
				assertEquals(
						"   'count_of_step'   "
								+ i, correctGraph.getProperties()
								.getCountOfSegment(), readGraph.getProperties()
								.getCountOfSegment());
				assertEquals(
						"   'color'   "
								+ i, correctGraph.getProperties().getColor(),
						readGraph.getProperties().getColor());
				assertEquals(
						"   'isPaint'   "
								+ i, correctGraph.isGraphPaint(), readGraph
								.isGraphPaint());
				i++;
			}
		} catch (IOException ex) {
		} catch (ParserConfigurationException ex) {
		} catch (SAXException ex) {
		}
	}

}
