package org.amse.chekh.paint_graph.view;

import java.awt.*;
import java.net.URL;
import java.net.URLClassLoader;

import javax.swing.*;
import javax.swing.border.LineBorder;

import org.amse.chekh.paint_graph.abstact_view.IAbstractView;
import org.amse.chekh.paint_graph.abstact_view.IEvent;
import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.amse.chekh.paint_graph.abstact_view.IListener;

/**
 *   
 * 
 * @author 
 */
public class ListForFunctions extends JList implements IListener {

	private static final long serialVersionUID = 1L;

	private IAbstractView myAbstractView;

	/**
	 *    
	 * 
	 * @param model
	 */
	public ListForFunctions(IAbstractView abstractView) {
		super(new DefaultListModel());
		this.myAbstractView = abstractView;
		this.myAbstractView.addListener(this);
		this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.setCellRenderer(new FunctionListCellRenderer());
	}

	public void processEvent(IEvent e) {
		if (!e.getTextOfEvent().equals("changeGrapgProperties")) {
			this.repaint();
		}
	}

	public void setAbstractView(IAbstractView newView) {
		this.myAbstractView = newView;
		this.myAbstractView.addListener(this);
	}

	/**
	 *         
	 */
	private class FunctionListCellRenderer extends JPanel implements
			ListCellRenderer {

		private static final long serialVersionUID = 1L;

		/**
		 *        
		 */
		private JLabel label = new JLabel();

		/**
		 *       ( )
		 */
		private JCheckBox checkBox = new JCheckBox();

		/**
		 *      
		 */
		private FunctionListCellRenderer() {
			this.setLayout(new BorderLayout());
			this.add(this.checkBox, BorderLayout.WEST);
			this.add(this.label, BorderLayout.CENTER);
		}

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			IGraph graph = (IGraph) value;
			label.setIcon(this.getIcon(graph));
			label.setText("Y = " + graph.getFunction().getText());
			label.setIconTextGap(10);
			label.setHorizontalTextPosition(SwingConstants.RIGHT);
			checkBox.setSelected(graph.isGraphPaint());
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}
			if (cellHasFocus) {
				setBorder(new LineBorder(Color.BLACK));
			} else {
				setBorder(null);
			}
			return this;
		}

		private URL getURL(String fileName) {
			URLClassLoader urlLoader = (URLClassLoader) this.getClass()
					.getClassLoader();
			URL fileLocation = urlLoader.findResource(fileName);
			return fileLocation;
		}

		private ImageIcon getIcon(IGraph graph) {
			Color color = graph.getProperties().getColor();
			if (color == Color.BLACK) {
				return new ImageIcon(
						getURL("org/amse/chekh/paint_graph/icon/Black.gif"));
			} else if (color == Color.RED) {
				return new ImageIcon(
						getURL("org/amse/chekh/paint_graph/icon/Red.gif"));
			} else if (color == Color.BLUE) {
				return new ImageIcon(
						getURL("org/amse/chekh/paint_graph/icon/Blue.gif"));
			} else if (color == Color.GREEN) {
				return new ImageIcon(
						getURL("org/amse/chekh/paint_graph/icon/Green.gif"));
			} else {
				return null;
			}
		}

	}

}
