package org.amse.chekh.paint_graph.view;

import java.awt.*;

import java.util.List;

import javax.swing.JPanel;

import org.amse.chekh.paint_graph.abstact_view.IAbstractView;
import org.amse.chekh.paint_graph.abstact_view.IEvent;
import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.amse.chekh.paint_graph.abstact_view.IGraphProperties;
import org.amse.chekh.paint_graph.abstact_view.IListener;
import org.amse.chekh.paint_graph.model.IFunction;

/**
 * ,    
 * 
 * @author 
 */
public class PaintPanel extends JPanel implements IListener {

	private boolean myRectPaint = false;

	private Point myBegin;

	private Point myEnd;

	private static final long serialVersionUID = 1L;

	/**
	 *   ...
	 */
	private final double myEpsilon = 1E-1;

	/**
	 *    
	 */
	private IAbstractView myAbstractView;

	/**
	 *    X
	 */
	private double myMinVisibleX;

	/**
	 *    X
	 */
	private double myMaxVisibleX;

	/**
	 *    Y
	 */
	private double myMinVisibleY;

	/**
	 *    Y
	 */
	private double myMaxVisibleY;

	/**
	 *     
	 * 
	 * @param paintGraphModel -
	 *              
	 */
	public PaintPanel(IAbstractView abstractView) {
		super();
		this.myAbstractView = abstractView;
		this.myAbstractView.addListener(this);
		this.myMinVisibleX = -10.0;
		this.myMaxVisibleX = 10.0;
		this.myMinVisibleY = -10.0;
		this.myMaxVisibleY = 10.0;
		this.setVisible(true);
	}

	/**
	 *     
	 * 
	 * @param newModel -
	 *             
	 */
	public void setAbstractView(IAbstractView newAbstractView) {
		this.myAbstractView = newAbstractView;
		this.myAbstractView.addListener(this);
	}

	public IAbstractView getAbstractView() {
		return this.myAbstractView;
	}

	public void processEvent(IEvent e) {
		this.repaint();
	}

	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		super.paintComponent(g);
		List<IGraph> graphs = myAbstractView.graphs();
		int xMax = this.getWidth();
		int yMax = this.getHeight();
		int x0, y0;
		if (this.myMinVisibleX >= 0) {
			x0 = 5;
		} else if (this.myMaxVisibleX <= 0) {
			x0 = xMax - 5;
		} else {
			x0 = (int) (xMax * this.myMinVisibleX / (this.myMinVisibleX - this.myMaxVisibleX));
			if (x0 < 5) {
				x0 = 5;
			}
			if (x0 > xMax - 5) {
				x0 = xMax - 5;
			}
		}
		if (this.myMaxVisibleY <= 0) {
			y0 = 5;
		} else if (this.myMinVisibleY >= 0) {
			y0 = yMax - 5;
		} else {
			y0 = (int) (yMax * this.myMaxVisibleY / (this.myMaxVisibleY - this.myMinVisibleY));
			if (y0 < 5) {
				y0 = 5;
			}
			if (y0 > yMax - 5) {
				y0 = yMax - 5;
			}
		}

		/*   */
		g2.setColor(Color.BLACK);
		if (this.myMinVisibleY > 0 || this.myMaxVisibleY < 0) {
			g2.setStroke(new BasicStroke(1f, BasicStroke.CAP_BUTT,
					BasicStroke.JOIN_MITER, 1f, new float[] { 3f, 1f }, 0f));
		} else {
			g2.setStroke(new BasicStroke(1.0f));
		}
		g2.drawLine(0, y0, xMax, y0); /*  X */
		g2.drawLine(xMax, y0, xMax - 5, y0 - 3);
		g2.drawLine(xMax, y0, xMax - 5, y0 + 3);
		if (this.myMinVisibleX > 0 || this.myMaxVisibleX < 0) {
			g2.setStroke(new BasicStroke(1f, BasicStroke.CAP_BUTT,
					BasicStroke.JOIN_MITER, 1f, new float[] { 3f, 1f }, 0f));
		} else {
			g2.setStroke(new BasicStroke(1.0f));
		}
		g2.drawLine(x0, 0, x0, yMax); /*  Y */
		g2.drawLine(x0, 0, x0 - 3, 5);
		g2.drawLine(x0, 0, x0 + 3, 5);

		double deltaX = this.myMaxVisibleX - this.myMinVisibleX;
		double deltaY = this.myMaxVisibleY - this.myMinVisibleY;
		double kx = xMax / deltaX; /*     X */
		double ky = yMax / deltaY; /*     Y */
		for (IGraph graph : graphs) {
			if (!graph.isGraphPaint() || !graph.getFunction().isValid()) {
				continue;
			}
			if (graph.isGraphSelected()) {
				g2.setStroke(new BasicStroke(3.0f));
			} else {
				g2.setStroke(new BasicStroke(1.0f));
			}
			IGraphProperties properties = graph.getProperties();
			IFunction function = graph.getFunction();
			g2.setColor(properties.getColor());
			boolean isPaintLine; /*     */
			double minX = Math.max(myMinVisibleX, properties.getMinimumX());
			double maxX = Math.min(myMaxVisibleX, properties.getMaximumX());
			if (maxX <= minX) {
				continue;
			}
			int countOfSegment = Math.min(xMax, properties.getCountOfSegment());
			double x, y;
			int X1, X2, Y1, Y2;
			boolean isY1NaN, isY2NaN;
			x = minX;
			y = function.evaluate(x);
			X2 = (int) Math.round((x - this.myMinVisibleX) * kx);
			if (Double.isNaN(y)) {
				isY2NaN = true;
				Y2 = 0;
			} else {
				isY2NaN = false;
				Y2 = (int) Math.round((this.myMaxVisibleY - y) * ky);
			}
			if (!isY2NaN && Y2 >= 0 && Y2 <= yMax) {
				isPaintLine = true;
			} else {
				isPaintLine = false;
			}
			double delta = (maxX - minX) / countOfSegment;
			for (int i = 1; i <= countOfSegment; i++) {
				X1 = X2;
				Y1 = Y2;
				isY1NaN = isY2NaN;
				x = minX + i * delta;
				y = function.evaluate(x);
				X2 = (int) Math.round((x - this.myMinVisibleX) * kx);
				if (Double.isNaN(y)) {
					isY2NaN = true;
					Y2 = 0;
				} else {
					isY2NaN = false;
					Y2 = (int) Math.round((this.myMaxVisibleY - y) * ky);
				}
				if (isPaintLine) {
					if (!isY2NaN && Y2 >= 0 && Y2 <= yMax) {
						g2.drawLine(X1, Y1, X2, Y2);
					} else {
						if (!isY2NaN) {
							g2.drawLine(X1, Y1, X2, Y2);
						}
						isPaintLine = false;
					}
				} else {
					if (!isY2NaN && Y2 >= 0 && Y2 <= yMax) {
						if (!isY1NaN) {
							g2.drawLine(X1, Y1, X2, Y2);
						}
						isPaintLine = true;
					}
				}
			}
		}
		if (this.myRectPaint) {
			g2.setColor(Color.BLACK);
			int x1 = Math.min(this.myBegin.x, this.myEnd.x);
			if (x1 < 0) {
				x1 = 0;
			}
			int x2 = Math.max(this.myBegin.x, this.myEnd.x);
			if (x2 > this.getWidth()) {
				x2 = this.getWidth() - 1;
			}
			int y1 = Math.min(this.myBegin.y, this.myEnd.y);
			if (y1 < 0) {
				y1 = 0;
			}
			int y2 = Math.max(this.myBegin.y, this.myEnd.y);
			if (y2 > this.getHeight()) {
				y2 = this.getHeight() - 1;
			}
			g2.setStroke(new BasicStroke(1f, BasicStroke.CAP_BUTT,
					BasicStroke.JOIN_MITER, 1f, new float[] { 3f, 1f }, 0f));
			g2.drawRect(x1, y1, x2 - x1, y2 - y1);
		}
		g2.dispose();
	}

	/**
	 * ,        - . 
	 * ,   .
	 * 
	 * @param point -
	 *                ,    
	 */
	public void updateGraphSelection(Point point) {
		boolean isChange = false;
		int xMax = getWidth();
		int yMax = getHeight();
		double deltaX = myMaxVisibleX - myMinVisibleX;
		double deltaY = myMaxVisibleY - myMinVisibleY;
		double kx = xMax / deltaX; /*     X */
		double ky = yMax / deltaY; /*     Y */
		double x = point.getX() / kx + myMinVisibleX;
		double y = myMaxVisibleY - point.getY() / ky;
		for (IGraph graph : myAbstractView.graphs()) {
			double Y = graph.getFunction().evaluate(x);
			if (!Double.isNaN(Y) && Math.abs(Y - y) < this.myEpsilon) {
				if (!graph.isGraphSelected()) {
					isChange = true;
					this.myAbstractView.changeGraphSelected(graph);
				}
			} else {
				if (graph.isGraphSelected()) {
					isChange = true;
					this.myAbstractView.changeGraphSelected(graph);
				}
			}
		}
		if (isChange) {
			this.paintComponent(this.getGraphics());
		}
	}

	public void deselectedAllGraph() {
		boolean isChange = false;
		for (IGraph graph : myAbstractView.graphs()) {
			if (graph.isGraphSelected()) {
				isChange = true;
				this.myAbstractView.changeGraphSelected(graph);
			}
		}
		if (isChange) {
			this.paintComponent(this.getGraphics());
		}
	}

	public void changePaintRectangle() {
		int xMax = this.getWidth();
		int yMax = this.getHeight();
		double deltaX = myMaxVisibleX - myMinVisibleX;
		double deltaY = myMaxVisibleY - myMinVisibleY;
		double kx = xMax / deltaX;
		double ky = yMax / deltaY;
		int minX = Math.min(this.myBegin.x, this.myEnd.x);
		int maxX = Math.max(this.myBegin.x, this.myEnd.x);
		int minY = Math.min(this.myBegin.y, this.myEnd.y);
		int maxY = Math.max(this.myBegin.y, this.myEnd.y);
		this.myMaxVisibleX = maxX / kx + this.myMinVisibleX;
		this.myMinVisibleX = minX / kx + this.myMinVisibleX;
		this.myMinVisibleY = this.myMaxVisibleY - maxY / ky;
		this.myMaxVisibleY = this.myMaxVisibleY - minY / ky;
		this.repaint();
	}

	public void changePaintRectangle(double scale) {
		if (this.myMaxVisibleX * this.myMinVisibleX > 0) {
			if (this.myMaxVisibleX > 0) {
				this.myMinVisibleX /= scale;
				this.myMaxVisibleX *= scale;
			} else {
				this.myMinVisibleX *= scale;
				this.myMaxVisibleX /= scale;
			}
		} else {
			this.myMinVisibleX *= scale;
			this.myMaxVisibleX *= scale;
		}
		if (this.myMaxVisibleY * this.myMinVisibleY > 0) {
			if (this.myMaxVisibleY > 0) {
				this.myMinVisibleY /= scale;
				this.myMaxVisibleY *= scale;
			} else {
				this.myMinVisibleY *= scale;
				this.myMaxVisibleY /= scale;
			}
		} else {
			this.myMinVisibleY *= scale;
			this.myMaxVisibleY *= scale;
		}

		this.repaint();
	}

	public double getMinimunVisibleX() {
		return this.myMinVisibleX;
	}

	public double getMinimunVisibleY() {
		return this.myMinVisibleY;
	}

	public double getMaximunVisibleX() {
		return this.myMaxVisibleX;
	}

	public double getMaximunVisibleY() {
		return this.myMaxVisibleY;
	}

	public void setMinimunVisibleX(double minX) {
		this.myMinVisibleX = minX;
	}

	public void setMinimunVisibleY(double minY) {
		this.myMinVisibleY = minY;
	}

	public void setMaximunVisibleX(double maxX) {
		this.myMaxVisibleX = maxX;
	}

	public void setMaximunVisibleY(double maxY) {
		this.myMaxVisibleY = maxY;
	}

	public void setBeginPointForScale(Point begin) {
		this.myBegin = begin;
	}

	public void setEndPointForScale(Point end) {
		this.myEnd = end;
	}

	public void setRectPaint(boolean isRectPaint) {
		this.myRectPaint = isRectPaint;
	}

}
