package org.amse.chekh.paint_graph.view.dialogs;

import java.awt.*;
import java.awt.event.*;

import java.util.List;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.amse.chekh.paint_graph.exception.DifferencialException;
import org.amse.chekh.paint_graph.model.IFunction;
import org.amse.chekh.paint_graph.model.impl.FunctionFactory;

public class DifferencialDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	private JComboBox myFunctions = new JComboBox();

	private JTextField myDifferencialField = new JTextField(20);

	private IFunction myDiffFunction = null;

	private JButton myOkButton = new JButton(" ");
	
	private JLabel myError = new JLabel();

	public DifferencialDialog(JFrame owner) {
		super(owner, " ", true);

		this.setLocation(new Point((int) (owner.getLocation().getX() + 100),
				(int) (owner.getLocation().getY() + 100)));

		this.myFunctions.setEditable(true);
		this.myFunctions.setMaximumRowCount(5);

		this.setResizable(false);

		this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent we) {
				DifferencialDialog.this.myDiffFunction = null;
				DifferencialDialog.this.setVisible(false);
			}

		});

		this.myDifferencialField.setEditable(false);

		final JTextField editor = (JTextField) this.myFunctions.getEditor()
				.getEditorComponent();
		editor.getDocument().addDocumentListener(new DocumentListener() {

			public void removeUpdate(DocumentEvent e) {
				DifferencialDialog.this.create(editor.getText());
			}

			public void insertUpdate(DocumentEvent e) {
				DifferencialDialog.this.create(editor.getText());
			}

			public void changedUpdate(DocumentEvent e) {
				DifferencialDialog.this.create(editor.getText());
			}

		});

		this.myFunctions.setRenderer(new DefaultListCellRenderer() {

			private static final long serialVersionUID = 1L;

			@Override
			public Component getListCellRendererComponent(JList list,
					Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				return super.getListCellRendererComponent(list,
						"Y(x)=" + value, index, isSelected, cellHasFocus);
			}

		});

		Box diff = Box.createHorizontalBox();
		diff.setBorder(new CompoundBorder(BorderFactory.createEmptyBorder(5, 5,
				5, 5), new TitledBorder("")));
		diff.add(new JLabel("Y(x) = "));
		diff.add(Box.createHorizontalStrut(5));
		diff.add(this.myDifferencialField);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		JPanel grid = new JPanel(new GridLayout(1, 2, 5, 0));

		JButton cancel = new JButton("Cancel");
		grid.add(this.myOkButton);
		grid.add(cancel);
		this.myOkButton.addActionListener(new EnterActionListener());
		cancel.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				DifferencialDialog.this.myDiffFunction = null;
				DifferencialDialog.this.setVisible(false);
			}

		});

		buttonPanel.add(grid);

		Box main = Box.createVerticalBox();
		main.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
		main.add(this.myFunctions);
		main.add(Box.createVerticalStrut(12));
		JPanel p = new JPanel(new BorderLayout());
		p.add(this.myError, BorderLayout.CENTER);
		this.myError.setPreferredSize(this.myDifferencialField.getPreferredSize());
		this.myError.setMaximumSize(this.myError.getPreferredSize());
		this.myError.setMinimumSize(this.myError.getPreferredSize());
		main.add(p);
		main.add(Box.createVerticalStrut(12));
		main.add(diff);
		main.add(Box.createVerticalStrut(12));
		main.add(buttonPanel);
		buttonPanel.setBackground(main.getBackground());
		this.setContentPane(main);

		InputMap im = this.getRootPane().getInputMap(
				JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap am = this.getRootPane().getActionMap();
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "Enter");
		am.put("Enter", new AbstractAction() {

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (DifferencialDialog.this.myOkButton.isEnabled()) {
					DifferencialDialog.this.setVisible(false);
				}
			}

		});
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "Escape");
		am.put("Escape", new AbstractAction() {

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				DifferencialDialog.this.myDiffFunction = null;
				DifferencialDialog.this.setVisible(false);
			}

		});

		pack();

	}

	public IFunction createDifferencial(List<IGraph> graphs) {
		this.myFunctions.setModel(new DefaultComboBoxModel());
		this.myDifferencialField.setText("");
		for (IGraph g : graphs) {
			this.myFunctions.addItem(g.getFunction().getText());
		}
		if (graphs.size() == 0) {
			this.myOkButton.setEnabled(false);
		} else {
			this.myOkButton.setEnabled(true);
		}
		this.setVisible(true);
		return this.myDiffFunction;
	}

	private void create(String s) {
		IFunction function = FunctionFactory.createFunction(s);
		if (!function.isValid()) {
			this.myError.setText("<html><font color=red>  !</html>");
			this.myOkButton.setEnabled(false);
		} else {
			try {
				this.myDiffFunction = function.getDifferencial();
				this.myDifferencialField.setText(this.myDiffFunction.getText());
				this.myError.setText("");
				this.myOkButton.setEnabled(true);
			} catch (DifferencialException ex) {
				this.myError.setText("<html><font color=red>   !</html>");
				this.myOkButton.setEnabled(false);
			}
		}

	}

	private class EnterActionListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			DifferencialDialog.this.setVisible(false);
		}

	}

}
