package org.amse.chekh.paint_graph.view.dialogs;

import java.awt.*;
import java.awt.event.*;

import java.util.List;

import javax.swing.*;
import javax.swing.border.*;

import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.amse.chekh.paint_graph.model.IFunction;
import org.amse.chekh.paint_graph.model.impl.FunctionFactory;

public class IntegralDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	private JComboBox myFunctions = new JComboBox();

	private JCheckBox myAbsCalculate = new JCheckBox(
			"   ()");

	private JTextField myMinimumXField = new JTextField(10);

	private JTextField myMaximumXField = new JTextField(10);

	private JSpinner mySpinner = new JSpinner(new SpinnerNumberModel(10, 1,
			20000, 10));

	private JTextField myIntegralCalculateField = new JTextField(20);

	public IntegralDialog(JFrame owner) {
		super(owner, " ", true);

		this.setLocation(new Point((int) (owner.getLocation().getX() + 100),
				(int) (owner.getLocation().getY() + 100)));

		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		this.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
					IntegralDialog.this.setVisible(false);
				}
			}

		});

		this.myIntegralCalculateField.setEditable(false);
		this.myFunctions.setEditable(true);
		this.myFunctions.setMaximumRowCount(5);
		this.myFunctions.setPreferredSize(new Dimension(200, 20));
		this.myFunctions.setMaximumSize(this.myFunctions.getPreferredSize());
		this.myFunctions.setMinimumSize(this.myFunctions.getPreferredSize());

		JPanel parametrs = new JPanel(new GridLayout(6, 1, 0, 5));
		parametrs.setBorder(new CompoundBorder(BorderFactory.createEmptyBorder(
				5, 5, 5, 5), new TitledBorder("")));

		Box function = Box.createHorizontalBox();
		JLabel l1 = new JLabel("F(x) = ");
		function.add(l1);
		function.add(Box.createHorizontalStrut(5));
		function.add(this.myFunctions);

		parametrs.add(function);

		parametrs.add(this.myAbsCalculate);

		Box minXBox = Box.createHorizontalBox();
		l1 = new JLabel(" X");
		minXBox.add(l1);
		minXBox.add(Box.createHorizontalStrut(5));
		minXBox.add(this.myMinimumXField);

		Box maxXBox = Box.createHorizontalBox();
		JLabel l2 = new JLabel(" X");
		maxXBox.add(l2);
		maxXBox.add(Box.createHorizontalStrut(5));
		maxXBox.add(this.myMaximumXField);

		Box segmentBox = Box.createHorizontalBox();
		JLabel l3 = new JLabel(" ");
		segmentBox.add(l3);
		segmentBox.add(Box.createHorizontalStrut(5));
		segmentBox.add(this.mySpinner);

		l1.setPreferredSize(l3.getPreferredSize());
		l1.setMinimumSize(l3.getPreferredSize());
		l1.setMaximumSize(l3.getPreferredSize());
		l2.setPreferredSize(l3.getPreferredSize());
		l2.setMinimumSize(l3.getPreferredSize());
		l2.setMaximumSize(l3.getPreferredSize());
		l3.setMinimumSize(l3.getPreferredSize());
		l3.setMaximumSize(l3.getPreferredSize());

		parametrs.add(minXBox);
		parametrs.add(maxXBox);
		parametrs.add(segmentBox);

		Box sum = Box.createHorizontalBox();
		sum.add(new JLabel(""));
		sum.add(Box.createHorizontalStrut(5));
		sum.add(this.myIntegralCalculateField);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		JPanel grid = new JPanel(new GridLayout(1, 2, 5, 0));

		JButton calculate = new JButton("");
		JButton cancel = new JButton("Cancel");
		grid.add(calculate);
		grid.add(cancel);
		calculate.addActionListener(new CalculateActionListener());
		cancel.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				IntegralDialog.this.setVisible(false);
			}

		});

		buttonPanel.add(grid);

		Box main = Box.createVerticalBox();
		main.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
		main.add(parametrs);
		main.add(Box.createVerticalStrut(12));
		main.add(sum);
		main.add(Box.createVerticalStrut(12));
		main.add(buttonPanel);
		buttonPanel.setBackground(main.getBackground());
		this.setContentPane(main);

		InputMap im = this.getRootPane().getInputMap(
				JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap am = this.getRootPane().getActionMap();
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "Enter");
		am.put("Enter", new AbstractAction() {

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				IntegralDialog.this.calculate();
			}

		});
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "Escape");
		am.put("Escape", new AbstractAction() {

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				IntegralDialog.this.setVisible(false);
			}

		});

		pack();

	}

	public void integrate(List<IGraph> graphs) {
		this.myIntegralCalculateField.setText("");
		this.myFunctions.setModel(new DefaultComboBoxModel());
		for (IGraph g : graphs) {
			this.myFunctions.addItem(g.getFunction().getText());
		}
		this.setVisible(true);
	}

	private void calculate() {
		IFunction function = FunctionFactory.createFunction((String) this.myFunctions
				.getSelectedItem());
		if (!function.isValid()) {
			JOptionPane.showMessageDialog(this, "  !",
					"", JOptionPane.ERROR_MESSAGE);
		} else {
			try {
				double xMin = Double
						.parseDouble(this.myMinimumXField.getText());
				double xMax = Double
						.parseDouble(this.myMaximumXField.getText());
				if (xMin >= xMax) {
					JOptionPane.showMessageDialog(this,
							" X     X!",
							"", JOptionPane.ERROR_MESSAGE);
					this.myMinimumXField.requestFocus();
				} else {
					int countOfSegment = Integer.parseInt(this.mySpinner
							.getValue().toString());
					boolean isAbsCalculate = this.myAbsCalculate.isSelected();
					double x = xMin;
					double delta = (xMax - xMin) / countOfSegment;
					double y2 = function.evaluate(x);
					double y1;
					double sum = 0.0;
					if (Double.isNaN(y2)) {
						JOptionPane.showMessageDialog(this,
								"       " + x,
								"", JOptionPane.ERROR_MESSAGE);
					} else {
						for (int i = 1; i <= countOfSegment; i++) {
							y1 = y2;
							x = xMin + delta * i;
							y2 = function.evaluate(x);
							if (Double.isNaN(y2)) {
								JOptionPane.showMessageDialog(this,
										"       "
												+ x, "",
										JOptionPane.ERROR_MESSAGE);
								return;
							}
							if (y1 * y2 >= 0.0) {
								if (isAbsCalculate) {
									sum += delta * Math.abs(y1 + y2) / 2;
								} else {
									sum += delta * (y1 + y2) / 2;
								}
							} else {
								double h1 = delta / (1 + y2 / y1);
								double h2 = delta - h1;
								if (isAbsCalculate) {
									sum += Math.abs(y1) * h1 / 2 + Math.abs(y2)
											* h2 / 2;
								} else {
									sum += y1 * h1 / 2 + y2 * h2 / 2;
								}
							}
						}
						this.myIntegralCalculateField.setText("" + sum);
					}
				}
			} catch (NumberFormatException ex) {
				JOptionPane.showMessageDialog(this,
						"     !", "",
						JOptionPane.ERROR_MESSAGE);
				this.myMinimumXField.requestFocus();
				this.myMinimumXField.selectAll();
			}
		}
	}

	private class CalculateActionListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			IntegralDialog.this.calculate();
		}

	}

}
