package org.amse.chekh.paint_graph.view.dialogs;

import java.awt.*;
import java.awt.event.*;
import java.util.List;

import javax.swing.*;
import javax.swing.border.*;

import org.amse.chekh.paint_graph.abstact_view.IGraph;
import org.amse.chekh.paint_graph.exception.DifferencialException;
import org.amse.chekh.paint_graph.model.IFunction;

public class TangentOrNormalDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	private JComboBox myFunctions = new JComboBox();

	private JTextField myFieldForPoint = new JTextField(20);

	private JRadioButton myTangent = new JRadioButton("");

	private JRadioButton myNormal = new JRadioButton("");

	private String myFunction;

	public TangentOrNormalDialog(JFrame owner) {
		super(owner, " / ", true);

		this.setLocation(new Point((int) (owner.getLocation().getX() + 100),
				(int) (owner.getLocation().getY() + 100)));

		this.setResizable(false);

		this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent we) {
				TangentOrNormalDialog.this.myFunction = null;
				TangentOrNormalDialog.this.setVisible(false);
			}

		});
		this.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
					TangentOrNormalDialog.this.myFunction = null;
					TangentOrNormalDialog.this.setVisible(false);
				}
			}

		});

		ActionListener enterListener = new EnterActionListener();
		this.myFieldForPoint.addActionListener(enterListener);
		this.myFunctions.setRenderer(new DefaultListCellRenderer() {

			private static final long serialVersionUID = 1L;

			@Override
			public Component getListCellRendererComponent(JList list,
					Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				return super.getListCellRendererComponent(list, "Y(x)="
						+ ((IFunction) value).getText(), index, isSelected,
						cellHasFocus);
			}

		});

		Box x = Box.createHorizontalBox();
		x.setBorder(new CompoundBorder(BorderFactory.createEmptyBorder(5, 5, 5,
				5), new TitledBorder("  / ")));
		x.add(new JLabel("X = "));
		x.add(Box.createHorizontalStrut(5));
		x.add(this.myFieldForPoint);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		JPanel grid = new JPanel(new GridLayout(1, 2, 5, 0));

		JButton ok = new JButton(" ");
		JButton cancel = new JButton("Cancel");
		grid.add(ok);
		grid.add(cancel);
		ok.addActionListener(enterListener);
		cancel.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				TangentOrNormalDialog.this.myFunction = null;
				TangentOrNormalDialog.this.setVisible(false);
			}

		});

		buttonPanel.add(grid);

		JPanel tangentOrNormal = new JPanel(new GridLayout(1, 2, 5, 0));
		tangentOrNormal.setBorder(new CompoundBorder(BorderFactory
				.createEmptyBorder(5, 5, 5, 5), new TitledBorder("")));
		ButtonGroup myTangentOrNormal = new ButtonGroup();
		this.myTangent.setSelected(true);
		myTangentOrNormal.add(this.myTangent);
		myTangentOrNormal.add(this.myNormal);
		tangentOrNormal.add(this.myTangent);
		tangentOrNormal.add(this.myNormal);

		Box main = Box.createVerticalBox();
		main.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
		main.add(this.myFunctions);
		main.add(Box.createVerticalStrut(12));
		main.add(x);
		main.add(Box.createVerticalStrut(12));
		main.add(tangentOrNormal);
		main.add(Box.createVerticalStrut(12));
		main.add(buttonPanel);
		buttonPanel.setBackground(main.getBackground());
		this.setContentPane(main);

		InputMap im = this.getRootPane().getInputMap(
				JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap am = this.getRootPane().getActionMap();
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "Enter");
		am.put("Enter", new AbstractAction() {

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				TangentOrNormalDialog.this.create();
			}

		});
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "Escape");
		am.put("Escape", new AbstractAction() {

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				TangentOrNormalDialog.this.myFunction = null;
				TangentOrNormalDialog.this.setVisible(false);
			}

		});

		pack();

	}

	public String createTangentOrNormal(List<IGraph> graphs) {
		this.myTangent.setSelected(true);
		this.myFieldForPoint.setText("");
		this.myFunctions.removeAllItems();
		for (IGraph g : graphs) {
			this.myFunctions.addItem(g.getFunction());
		}
		this.setVisible(true);
		return this.myFunction;
	}

	private void create() {
		try {
			IFunction function = (IFunction) this.myFunctions.getSelectedItem();
			double x = Double.parseDouble(this.myFieldForPoint.getText());
			IFunction diffFunction = function.getDifferencial();
			double k = diffFunction.evaluate(x);
			if (this.myNormal.isSelected()) {
				if (k == 0.0 || Double.isNaN(k)) {
					k = Double.NaN;
				} else {
					k = -1.0 / k;
				}
			}
			if (!Double.isNaN(k)) {
				double b = function.evaluate(x);
				if (!Double.isNaN(b)) {
					b -= k * x;
					StringBuffer sb = new StringBuffer();
					if (k != 0.0) {
						sb.append("" + k);
						sb.append("*x");
					}
					if (b > 0.0) {
						if (sb.length() > 0) {
							sb.append("+" + b);
						} else {
							sb.append("" + b);
						}
					} else if (b < 0.0) {
						sb.append("" + b);
					}
					if (sb.length() == 0) {
						this.myFunction = "0.0";
					} else {
						this.myFunction = sb.toString();
					}
					this.setVisible(false);
				} else {
					if (this.myTangent.isSelected()) {
						JOptionPane.showMessageDialog(this,
								"      !",
								"", JOptionPane.ERROR_MESSAGE);
					} else {
						JOptionPane.showMessageDialog(this,
								"      !",
								"", JOptionPane.ERROR_MESSAGE);
					}
				}
			} else {
				if (this.myTangent.isSelected()) {
					JOptionPane.showMessageDialog(this,
							"      !",
							"", JOptionPane.ERROR_MESSAGE);
				} else {
					JOptionPane.showMessageDialog(this,
							"      !",
							"", JOptionPane.ERROR_MESSAGE);
				}
			}
		} catch (NumberFormatException ex) {
			JOptionPane
					.showMessageDialog(
							this,
							"   .     ",
							"", JOptionPane.ERROR_MESSAGE);
			this.myFieldForPoint.requestFocus();
			this.myFieldForPoint.selectAll();
		} catch (DifferencialException ex) {
			JOptionPane.showMessageDialog(this, " y = "
					+ ((IFunction) this.myFunctions.getSelectedItem())
							.getText() + "  !", "",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	private class EnterActionListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			TangentOrNormalDialog.this.create();
		}
	}

}
